@@ -8,7 +8,7 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
    
@@ -17,7 +17,7 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
@@ -28,6 +28,7 @@
 
 #ifndef	INCLUDED_GS_CATEGORIES_H
 #define	INCLUDED_GS_CATEGORIES_H
+#include "GNUstepBase/GSVersionMacros.h"
 
 /* The following ifndef prevents the categories declared in this file being
  * seen in GNUstep code.  This is necessary because those category
@@ -49,6 +50,12 @@
 #include "GNUstepBase/GSObjCRuntime.h"
 #include "GNUstepBase/GNUstep.h"
 
+#if	defined(__cplusplus)
+extern "C" {
+#endif
+
+#if	OS_API_VERSION(GS_API_NONE,GS_API_LATEST)
+
 @class NSMutableSet;
 
 
@@ -103,6 +110,20 @@
 #define NSDebugMLog(format, args...)
 #endif /* DEBUG */
 
+#define GSOnceFLog(format, args...) \
+  do { static BOOL beenHere = NO; if (beenHere == NO) {\
+    NSString *fmt = GSDebugFunctionMsg( \
+        __PRETTY_FUNCTION__, __FILE__, __LINE__, format); \
+    beenHere = YES; \
+    NSLog(fmt , ## args); }} while (0)
+
+#define GSOnceMLog(format, args...) \
+  do { static BOOL beenHere = NO; if (beenHere == NO) {\
+    NSString *fmt = GSDebugMethodMsg( \
+        self, _cmd, __FILE__, __LINE__, format); \
+    beenHere = YES; \
+    NSLog(fmt , ## args); }} while (0)
+
 #ifdef GSWARN
 #define NSWarnLog(format, args...) \
     do { if (GSDebugSet(@"NoWarn") == NO) { \
@@ -331,9 +352,6 @@ GS_EXPORT void NSDecimalFromComponents(NSDecimal *result,
 
 GS_EXPORT BOOL GSDebugSet(NSString *level);
 
-GS_EXPORT NSString *GetEncodingName(NSStringEncoding availableEncodingValue);
-GS_EXPORT NSString *GSEncodingName(NSStringEncoding availableEncodingValue);
-
 GS_EXPORT NSThread *GSCurrentThread(void);
 GS_EXPORT NSMutableDictionary *GSCurrentThreadDictionary(void);
 
@@ -342,6 +360,12 @@ GS_EXPORT NSString *GSDebugMethodMsg(id obj, SEL sel, const char *file,
 GS_EXPORT NSString *GSDebugFunctionMsg(const char *func, const char *file,
 				       int line, NSString *fmt);
 
+#endif	/* OS_API_VERSION(GS_API_NONE,GS_API_NONE) */
+
+#if	defined(__cplusplus)
+}
+#endif
+
 #endif	/* GNUSTEP */
 
 #endif	/* INCLUDED_GS_CATEGORIES_H */
