@@ -9,7 +9,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 3 of the License, or (at your option) any later version.
+ * version 2 of the License, or (at your option) any later version.
  * 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,8 +26,12 @@
 #if	OS_API_VERSION(GS_API_NONE,GS_API_LATEST)
 
 #include <Foundation/NSObject.h>
+#include <Foundation/NSException.h>
 #include <Foundation/NSZone.h>
 
+/* To turn assertions on, define GSI_ARRAY_CHECKS */
+#define GSI_ARRAY_CHECKS 1
+
 #if	defined(__cplusplus)
 extern "C" {
 #endif
@@ -37,15 +41,14 @@ extern "C" {
 #define INLINE inline
 #endif
 
-/* To turn assertions on, define GSI_ARRAY_CHECKS */
 #ifdef	GSI_ARRAY_CHECKS
-#define	GSI_ARRAY_CHECK NSCAssert(array->count <= array->cap && array->old <= array->cap && array->old >= 1, NSInternalInconsistencyException)
+#define	GSI_ARRAY_CHECK NSCAssert(array->count <= array->cap && array->old <= array->cap, NSInternalInconsistencyException)
 #else
 #define	GSI_ARRAY_CHECK
 #endif
 
 /*
- This file should be INCLUDED in files wanting to use the GSIArray
+ *      This file should be INCLUDED in files wanting to use the GSIArray
  *	functions - these are all declared inline for maximum performance.
  *
  *	The file including this one may predefine some macros to alter
@@ -300,7 +303,7 @@ GSIArrayAddItemNoRetain(GSIArray array, GSIArrayItem item)
  */
 static INLINE unsigned
 GSIArraySearch(GSIArray array, GSIArrayItem item, 
-	NSComparisonResult (*sorter)(GSIArrayItem, GSIArrayItem))
+  NSComparisonResult (*sorter)(GSIArrayItem, GSIArrayItem))
 {
   unsigned int	upper = array->count;
   unsigned int	lower = 0;
@@ -333,7 +336,7 @@ GSIArraySearch(GSIArray array, GSIArrayItem item,
 
 static INLINE unsigned
 GSIArrayInsertionPosition(GSIArray array, GSIArrayItem item, 
-	NSComparisonResult (*sorter)(GSIArrayItem, GSIArrayItem))
+  NSComparisonResult (*sorter)(GSIArrayItem, GSIArrayItem))
 {
   unsigned int	index;
 
@@ -356,7 +359,7 @@ GSIArrayInsertionPosition(GSIArray array, GSIArrayItem item,
 #ifdef	GSI_ARRAY_CHECKS
 static INLINE void
 GSIArrayCheckSort(GSIArray array, 
-	NSComparisonResult (*sorter)(GSIArrayItem, GSIArrayItem))
+  NSComparisonResult (*sorter)(GSIArrayItem, GSIArrayItem))
 {
   unsigned int	i;
 
@@ -364,18 +367,21 @@ GSIArrayCheckSort(GSIArray array,
     {
 #ifdef	GSI_ARRAY_CHECKS
       NSCAssert(((*sorter)(array->ptr[i-1], array->ptr[i]) 
+        != NSOrderedDescending), NSInvalidArgumentException);
 #endif
-	         != NSOrderedDecending), NSInvalidArgumentException);
     }
 }
 #endif
 
 static INLINE void
 GSIArrayInsertSorted(GSIArray array, GSIArrayItem item, 
-	NSComparisonResult (*sorter)(GSIArrayItem, GSIArrayItem))
+  NSComparisonResult (*sorter)(GSIArrayItem, GSIArrayItem))
 {
   unsigned int	index;
 
+#ifdef	GSI_ARRAY_CHECKS
+  GSIArrayCheckSort(array, sorter);
+#endif
   index = GSIArrayInsertionPosition(array, item, sorter);
   GSIArrayInsertItem(array, item, index);
 #ifdef	GSI_ARRAY_CHECKS
@@ -385,10 +391,13 @@ GSIArrayInsertSorted(GSIArray array, GSIArrayItem item,
 
 static INLINE void
 GSIArrayInsertSortedNoRetain(GSIArray array, GSIArrayItem item,
-	NSComparisonResult (*sorter)(GSIArrayItem, GSIArrayItem))
+  NSComparisonResult (*sorter)(GSIArrayItem, GSIArrayItem))
 {
   unsigned int	index;
 
+#ifdef	GSI_ARRAY_CHECKS
+  GSIArrayCheckSort(array, sorter);
+#endif
   index = GSIArrayInsertionPosition(array, item, sorter);
   GSIArrayInsertItemNoRetain(array, item, index);
 #ifdef	GSI_ARRAY_CHECKS
