@@ -11,7 +11,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 3 of the License, or (at your option) any later version.
+ * version 2 of the License, or (at your option) any later version.
  * 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -563,8 +563,8 @@ GSIMapNodeForKey(GSIMapTable map, GSIMapKey key)
 #if     (GSI_MAP_KTYPES & GSUNION_INT)
 /*
  * Specialized lookup for the case where keys are known to be simple integer
- * or pointer values that are their own hash values and con be compared with
- * a test for integer equality.
+ * or pointer values that are their own hash values (when converted to unsigned
+ * integers) and can be compared with a test for integer equality.
  */
 static INLINE GSIMapNode 
 GSIMapNodeForSimpleKey(GSIMapTable map, GSIMapKey key)
@@ -576,9 +576,9 @@ GSIMapNodeForSimpleKey(GSIMapTable map, GSIMapKey key)
     {
       return 0;
     }
-  bucket = map->buckets + key.uint % map->bucketCount;
+  bucket = map->buckets + ((unsigned)key.addr) % map->bucketCount;
   node = bucket->firstNode;
-  while ((node != 0) && node->key.uint != key.uint)
+  while ((node != 0) && node->key.addr != key.addr)
     {
       node = node->nextInBucket;
     }
