@@ -7,7 +7,7 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
    
@@ -16,9 +16,10 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
 
    AutogsdocSource: Additions/GSLock.m
 
@@ -33,6 +34,12 @@
 #include <Foundation/Foundation.h>
 #endif
 
+#include "GNUstepBase/GSObjCRuntime.h"
+
+#if	defined(__cplusplus)
+extern "C" {
+#endif
+
 @class NSNotification;
 
 @interface	GSLazyLock : NSLock
@@ -49,6 +56,13 @@
 - (void) _becomeThreaded: (NSNotification*)n;
 @end
 
+/** Global lock to be used by classes when operating on any global
+    data that invoke other methods which also access global; thus,
+    creating the potential for deadlock. */
+GS_EXPORT NSRecursiveLock *gnustep_global_lock;
+
+#if	defined(__cplusplus)
+}
+#endif
+
 #endif	/* INCLUDED_GS_LOCK_H */
-
-
