@@ -11,7 +11,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 3 of the License, or (at your option) any later version.
+ * version 2 of the License, or (at your option) any later version.
  * 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,6 +23,10 @@
  * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02111 USA. */
 
+/* Need Foundation/NSObjCRuntime.h for type declarations.
+ */
+#import <Foundation/NSObjCRuntime.h>
+
 /*
  *	Definitions for bitmap mask of types of element in union.
  */
@@ -58,6 +62,7 @@
 #if	defined(GSUNION) && defined(GSUNION_TYPES)
 
 typedef	union {
+  NSUInteger    addr;
 #if	((GSUNION_TYPES) & GSUNION_OBJ)
   id		obj;
   NSObject	*nso;
