@@ -7,7 +7,7 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
    
@@ -16,9 +16,10 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
 
   AutogsdocSource: Additions/behavior.m
 
@@ -26,9 +27,16 @@
 
 #ifndef __behavior_h_GNUSTEP_BASE_INCLUDE
 #define __behavior_h_GNUSTEP_BASE_INCLUDE
+#include <GNUstepBase/GSVersionMacros.h>
 
 #include <GNUstepBase/GSObjCRuntime.h>
 
+#if	defined(__cplusplus)
+extern "C" {
+#endif
+
+#if	OS_API_VERSION(GS_API_NONE,GS_API_LATEST)
+
 /* Call this method from CLASS's +initialize method to add a behavior
    to CLASS.  A "behavior" is like a protocol with an implementation.
 
@@ -56,4 +64,10 @@ GS_EXPORT void behavior_class_add_methods (Class class,
 /* Set to non-zero if you want debugging messages on stderr. */
 GS_EXPORT void behavior_set_debug(int i);
 
+#endif	/* OS_API_VERSION(GS_API_NONE,GS_API_NONE) */
+
+#if	defined(__cplusplus)
+}
+#endif
+
 #endif /* __behavior_h_GNUSTEP_BASE_INCLUDE */
