@@ -7,7 +7,7 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
    
@@ -16,7 +16,7 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
    
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
@@ -24,8 +24,13 @@
 
 #ifndef __NSAutoreleasePool_h_GNUSTEP_BASE_INCLUDE
 #define __NSAutoreleasePool_h_GNUSTEP_BASE_INCLUDE
+#import	<GNUstepBase/GSVersionMacros.h>
 
-#include <Foundation/NSObject.h>
+#import	<Foundation/NSObject.h>
+
+#if	defined(__cplusplus)
+extern "C" {
+#endif
 
 @class NSAutoreleasePool;
 @class NSThread;
@@ -65,7 +70,8 @@ typedef struct autorelease_thread_vars
 /* Initialize an autorelease_thread_vars structure for a new thread.
    This function is called in NSThread each time an NSThread is created.
    TV should be of type `struct autorelease_thread_vars *' */
-#define init_autorelease_thread_vars(TV)  memset (TV, 0, sizeof (typeof (*TV)))
+#define init_autorelease_thread_vars(TV) \
+memset (TV, 0, sizeof (__typeof__ (*TV)))
 
 
 
@@ -213,6 +219,12 @@ typedef struct autorelease_array_list
  */
 - (id) autorelease;
 
+#if OS_API_VERSION(100400, GS_API_LATEST)
+/** Not implemented
+ */
+- (void) drain;
+#endif
+
 /**
  * Destroys the receiver (calls -dealloc).
  */
@@ -223,7 +235,7 @@ typedef struct autorelease_array_list
  */
 - (id) retain;
 
-#ifndef	NO_GNUSTEP
+#if OS_API_VERSION(GS_API_NONE, GS_API_NONE)
 /**
  * <p>
  *   Counts the number of times that the specified object occurs
@@ -298,4 +310,8 @@ typedef struct autorelease_array_list
 #endif
 @end
 
+#if	defined(__cplusplus)
+}
+#endif
+
 #endif /* __NSAutoreleasePool_h_GNUSTEP_BASE_INCLUDE */
