@@ -13,7 +13,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
    
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -62,7 +62,8 @@ extern "C" {
 @interface NSLock : NSObject <NSLocking, GCFinalization>
 {
 @private
-  void *_mutex;
+  void          *_mutex;
+  NSString      *_name;
 }
 
 /**
@@ -85,6 +86,16 @@ extern "C" {
  */
 - (void) unlock;
 
+#if OS_API_VERSION(100500,GS_API_LATEST) 
+/** Return the name of the receiver or nil of none has been set.
+ */
+- (NSString*) name;
+
+/** Sets the name of the receiver (for use in debugging).
+ */
+- (void) setName: (NSString*)name;
+#endif
+
 @end
 
 /**
@@ -98,6 +109,7 @@ extern "C" {
   void *_condition;
   void *_mutex;
   int   _condition_value;
+  NSString      *_name;
 }
 
 /**
@@ -163,6 +175,16 @@ extern "C" {
  */
 - (void) unlock;
 
+#if OS_API_VERSION(100500,GS_API_LATEST) 
+/** Return the name of the receiver or nil of none has been set.
+ */
+- (NSString*) name;
+
+/** Sets the name of the receiver (for use in debugging).
+ */
+- (void) setName: (NSString*)name;
+#endif
+
 @end
 
 
@@ -177,6 +199,7 @@ extern "C" {
 {
 @private
   void *_mutex;
+  NSString      *_name;
 }
 
 /**
@@ -200,6 +223,16 @@ extern "C" {
  */
 - (void) unlock;
 
+#if OS_API_VERSION(100500,GS_API_LATEST) 
+/** Return the name of the receiver or nil of none has been set.
+ */
+- (NSString*) name;
+
+/** Sets the name of the receiver (for use in debugging).
+ */
+- (void) setName: (NSString*)name;
+#endif
+
 @end
 
 #if OS_API_VERSION(GS_API_NONE, GS_API_NONE)
