@@ -8,7 +8,7 @@
    This file is part of the GNUstep Library.
    
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
    
@@ -17,20 +17,25 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
    
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
 */
 
-#ifndef _NSNumberFormatter_h__
-#define _NSNumberFormatter_h__
+#ifndef _NSNumberFormatter_h_GNUSTEP_BASE_INCLUDE
+#define _NSNumberFormatter_h_GNUSTEP_BASE_INCLUDE
+#import	<GNUstepBase/GSVersionMacros.h>
 
-#ifndef	STRICT_OPENSTEP
+#if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 
-#include <Foundation/NSObject.h>
-#include <Foundation/NSFormatter.h>
-#include <Foundation/NSDecimalNumber.h>
+#import	<Foundation/NSObject.h>
+#import	<Foundation/NSFormatter.h>
+#import	<Foundation/NSDecimalNumber.h>
+
+#if	defined(__cplusplus)
+extern "C" {
+#endif
 
 @class	NSString, NSAttributedString, NSDictionary;
 
@@ -295,5 +300,11 @@
 
 @end
 
+#if	defined(__cplusplus)
+}
 #endif
-#endif
+
+#endif	/* GS_API_MACOSX */
+
+#endif	/* _NSNumberFormatter_h_GNUSTEP_BASE_INCLUDE */
+
