@@ -9,7 +9,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
    
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -99,6 +99,16 @@ extern "C" {
 #if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 - (BOOL) scanDecimal: (NSDecimal*)value;
 #endif
+#if OS_API_VERSION(100500, GS_API_LATEST)
+/** Not implemented */
+- (BOOL) scanHexDouble: (double *)result;
+/** Not implemented */
+- (BOOL) scanHexFloat: (float *)result;
+/** Not implemented */
+- (BOOL) scanHexLongLong: (unsigned long long *)result;
+/** Not implemented */
+- (BOOL) scanInteger: (NSInteger *)value;
+#endif
 @end
 
 #if	defined(__cplusplus)
