@@ -16,7 +16,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
    
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -47,6 +47,12 @@ extern "C" {
 @class NSMutableArray;
 @class NSMutableDictionary;
 
+#if OS_API_VERSION(100500,GS_API_LATEST) 
+GS_EXPORT NSString *const NSGrammarCorrections;
+GS_EXPORT NSString *const NSGrammarRange;
+GS_EXPORT NSString *const NSGrammarUserDescription;
+#endif
+
 @interface NSSpellServer : NSObject
 {
 @private
@@ -126,6 +132,24 @@ findMisspelledWordInString: (NSString *)stringToCheck
 - (void) spellServer: (NSSpellServer *)sender
        didForgetWord: (NSString *)word
           inLanguage: (NSString *)language;
+
+#if OS_API_VERSION(100300,GS_API_LATEST) 
+/** Not implemented */
+- (NSArray *) spellServer: (NSSpellServer *)sender
+  suggestCompletionsForPartialWordRange: (NSRange)range
+  inString: (NSString *)string
+  language: (NSString *)language;
+#endif
+
+#if OS_API_VERSION(100500,GS_API_LATEST) 
+/** Not implemented */
+- (NSRange) spellServer: (NSSpellServer *)sender
+  checkGrammarInString: (NSString *)stringToCheck
+  language: (NSString *)language
+  details: (NSArray **)details;
+
+#endif
+
 @end
 
 #if     defined(__cplusplus)
