@@ -9,7 +9,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -87,37 +87,21 @@ sslError(int err)
 
   SSL_load_error_strings();
 
-  switch (err)
+  if (err == SSL_ERROR_SYSCALL)
     {
-      case SSL_ERROR_NONE:
-	str = @"No error: really helpful";
-	break;
-      case SSL_ERROR_ZERO_RETURN:
-	str = @"Zero Return error";
-	break;
-      case SSL_ERROR_WANT_READ:
-	str = @"Want Read Error";
-	break;
-      case SSL_ERROR_WANT_WRITE:
-	str = @"Want Write Error";
-	break;
-      case SSL_ERROR_WANT_X509_LOOKUP:
-	str = @"Want X509 Lookup Error";
-	break;
-      case SSL_ERROR_SYSCALL:
-        {
-	  NSError	*e = [NSError _last];
+      NSError	*e = [NSError _last];
+
+      str = [NSString stringWithFormat: @"Syscall error %d - %@",
+        [e code], [e description]];
+    }
+  else if (err == SSL_ERROR_NONE)
+    {
+      str = @"No error: really helpful";
+    }
+  else
+    {
+      str = [NSString stringWithFormat: @"%s", ERR_reason_error_string(err)];
 
-	  str = [NSString stringWithFormat: @"Syscall error %d - %@",
-	    [e code], [e description]];
-	}
-	break;
-      case SSL_ERROR_SSL:
-	str = @"SSL Error: really helpful";
-	break;
-      default:
-	str = @"Standard system error: really helpful";
-	break;
     }
   return str;
 }
@@ -270,10 +254,8 @@ sslError(int err)
 	      /*
 	       * Some other error ... not just a timeout or disconnect
 	       */
-	      NSLog(@"unable to accept SSL connection from %@:%@ - %@",
+	      NSWarnLog(@"unable to accept SSL connection from %@:%@ - %@",
 		address, service, sslError(err));
-
-	      ERR_print_errors_fp(stderr);
 	    }
 	  RELEASE(self);
 	  return NO;
@@ -369,7 +351,6 @@ sslError(int err)
       RELEASE(final);
       if (err != SSL_ERROR_NONE)
 	{
-	  if (err != SSL_ERROR_WANT_READ && err != SSL_ERROR_WANT_WRITE)
 	  if (err != SSL_ERROR_WANT_READ && err != SSL_ERROR_WANT_WRITE)
 	    {
 	      /*
@@ -377,7 +358,6 @@ sslError(int err)
 	       */
 	      NSLog(@"unable to make SSL connection to %@:%@ - %@",
 		address, service, sslError(err));
-	      ERR_print_errors_fp(stderr);
 	    }
 	  RELEASE(self);
 	  return NO;
