@@ -9,7 +9,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -908,20 +908,49 @@ static void MD5Transform (uint32_t buf[4], uint32_t const in[16])
  */
 @implementation NSError(GSCategories)
 
-#ifndef HAVE_STRERROR
-static const char *
-strerror(int eno)
+
+#if !defined(__MINGW32__)
+#if !defined(HAVE_STRERROR_R)
+#if defined(HAVE_STRERROR)
+static int
+strerror_r(int eno, char *buf, int len)
+{
+  const char *ptr;
+  int   result;
+
+  [gnustep_global_lock lock];
+  ptr = strerror(eno);
+  if (ptr == 0)
+    {
+      strncpy(buf, "unknown error number", len);
+      result = -1;
+    }
+  else
+    {
+      strncpy(buf, strerror(eno), len);
+      result = 0;
+    }
+  [gnustep_global_lock unlock];
+  return result;
+}
+#else
+static int
+strerror_r(int eno, char *buf, int len)
 {
   extern char  *sys_errlist[];
   extern int    sys_nerr;
 
   if (eno < 0 || eno >= sys_nerr)
     {
-      return("unknown error number");
+      strncpy(buf, "unknown error number", len);
+      return -1;
     }
-  return(sys_errlist[eno]);
+  strncpy(buf, sys_errlist[eno], len);
+  return 0;
 }
 #endif
+#endif
+#endif
 
 /*
  * Returns an NSError instance encapsulating the last system error.
@@ -959,10 +988,15 @@ strerror(int eno)
     nil];
 #else
   NSString	*message;
+  char          buf[BUFSIZ];
 
   /* FIXME ... not all are POSIX, should we use NSMachErrorDomain for some? */
   domain = NSPOSIXErrorDomain;
-  message = [NSString stringWithCString: strerror(code)
+  if (strerror_r(code, buf, BUFSIZ) < 0)
+    {
+      sprintf(buf, "%ld", code);
+    }
+  message = [NSString stringWithCString: buf
 			       encoding: [NSString defaultCStringEncoding]];
   /* FIXME ... can we do better localisation? */
   info = [NSMutableDictionary dictionaryWithObjectsAndKeys:
