@@ -10,7 +10,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,21 +22,23 @@
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 
-   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
+   $Date: 2008-06-12 11:44:00 +0100 (Thu, 12 Jun 2008) $ $Revision: 26630 $
    */
 
-#include "config.h"
-#include "GNUstepBase/preface.h"
-#include "Foundation/NSArray.h"
-#include "GNUstepBase/GSObjCRuntime.h"
-#include "Foundation/NSException.h"
-#include "Foundation/NSPortCoder.h"
-#include "Foundation/NSDebug.h"
-#include "Foundation/NSValue.h"
+#import "config.h"
+#import "GNUstepBase/preface.h"
+#import "Foundation/NSArray.h"
+#import "GNUstepBase/GSObjCRuntime.h"
+#import "Foundation/NSDictionary.h"
+#import "Foundation/NSEnumerator.h"
+#import "Foundation/NSException.h"
+#import "Foundation/NSPortCoder.h"
+#import "Foundation/NSDebug.h"
+#import "Foundation/NSValue.h"
 // For private method _decodeArrayOfObjectsForKey:
-#include "Foundation/NSKeyedArchiver.h"
+#import "Foundation/NSKeyedArchiver.h"
 
-#include "GSPrivate.h"
+#import "GSPrivate.h"
 
 static SEL	eqSel;
 static SEL	oaiSel;
@@ -712,6 +714,21 @@ static Class	GSInlineArrayClass;
     {
       [self _raiseRangeExceptionWithIndex: index from: _cmd];
     }
+  if (!anObject)
+    {
+      NSException  *exception;
+      NSDictionary *info;
+
+      info = [NSDictionary dictionaryWithObjectsAndKeys:
+	[NSNumber numberWithUnsignedInt: index], @"Index",
+        _contents_array[index], @"OLdObject",
+	self, @"Array", nil, nil];
+
+      exception = [NSException exceptionWithName: NSInvalidArgumentException
+	reason: @"Tried to replace object in array with nil"
+	userInfo: info];
+      [exception raise];
+    }
   /*
    *	Swap objects in order so that there is always a valid object in the
    *	array in case a retain or release causes an exception.
