@@ -15,7 +15,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -44,18 +44,18 @@
  *		attributes argument and use the values from the string.
  */
 
-#include "config.h"
-#include "GNUstepBase/preface.h"
-#include "GNUstepBase/GSLock.h"
-#include "Foundation/NSAttributedString.h"
-#include "Foundation/NSException.h"
-#include "Foundation/NSRange.h"
-#include "Foundation/NSDebug.h"
-#include "Foundation/NSArray.h"
-#include "Foundation/NSLock.h"
-#include "Foundation/NSThread.h"
-#include "Foundation/NSNotification.h"
-#include "Foundation/NSZone.h"
+#import "config.h"
+#import "GNUstepBase/preface.h"
+#import "GNUstepBase/GSLock.h"
+#import "Foundation/NSAttributedString.h"
+#import "Foundation/NSException.h"
+#import "Foundation/NSRange.h"
+#import "Foundation/NSDebug.h"
+#import "Foundation/NSArray.h"
+#import "Foundation/NSLock.h"
+#import "Foundation/NSThread.h"
+#import "Foundation/NSNotification.h"
+#import "Foundation/NSZone.h"
 
 #define		SANITY_CHECKS	0
 
