@@ -9,7 +9,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,14 +22,14 @@
    Boston, MA 02111 USA.
    */
 
-#include "config.h"
-#include "Foundation/NSSet.h"
-#include "Foundation/NSAutoreleasePool.h"
-#include "Foundation/NSException.h"
-#include "Foundation/NSUtilities.h"
-#include "Foundation/NSString.h"
-#include "Foundation/NSPortCoder.h"
-#include "Foundation/NSDebug.h"
+#import "config.h"
+#import "Foundation/NSSet.h"
+#import "Foundation/NSAutoreleasePool.h"
+#import "Foundation/NSEnumerator.h"
+#import "Foundation/NSException.h"
+#import "Foundation/NSString.h"
+#import "Foundation/NSPortCoder.h"
+#import "Foundation/NSDebug.h"
 
 
 #define	GSI_MAP_RETAIN_VAL(M, X)	
