@@ -9,7 +9,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,22 +22,26 @@
    Boston, MA 02111 USA.
 */
 
-#include "config.h"
-#include "Foundation/NSArray.h"
-#include "Foundation/NSString.h"
-#include "Foundation/NSException.h"
-#include "Foundation/NSValue.h"
-#include "Foundation/NSData.h"
-#include "Foundation/NSURL.h"
-#include "Foundation/NSURLHandle.h"
-#include "Foundation/NSNotification.h"
-#include "Foundation/NSRunLoop.h"
-#include "Foundation/NSByteOrder.h"
-#include "Foundation/NSLock.h"
-#include "Foundation/NSFileHandle.h"
-#include "Foundation/NSDebug.h"
-#include "GNUstepBase/GSMime.h"
-#include "GSPrivate.h"
+#import "config.h"
+#import "Foundation/NSArray.h"
+#import "Foundation/NSDictionary.h"
+#import "Foundation/NSEnumerator.h"
+#import "Foundation/NSString.h"
+#import "Foundation/NSException.h"
+#import "Foundation/NSValue.h"
+#import "Foundation/NSData.h"
+#import "Foundation/NSDictionary.h"
+#import "Foundation/NSEnumerator.h"
+#import "Foundation/NSURL.h"
+#import "Foundation/NSURLHandle.h"
+#import "Foundation/NSNotification.h"
+#import "Foundation/NSRunLoop.h"
+#import "Foundation/NSByteOrder.h"
+#import "Foundation/NSLock.h"
+#import "Foundation/NSFileHandle.h"
+#import "Foundation/NSDebug.h"
+#import "GNUstepBase/GSMime.h"
+#import "GSPrivate.h"
 
 GS_EXPORT NSString * const GSTelnetNotification;
 GS_EXPORT NSString * const GSTelnetErrorKey;
