@@ -9,7 +9,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
    
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,14 +22,15 @@
    Boston, MA 02111 USA.
    */ 
 
-#include "GSURLPrivate.h"
-#include "Foundation/NSDictionary.h"
-#include "Foundation/NSScanner.h"
-#include "Foundation/NSSet.h"
-#include "Foundation/NSDebug.h"
-#include "Foundation/NSValue.h"
-#include "GNUstepBase/GSLock.h"
-#include "GNUstepBase/GSMime.h"
+#import "GSURLPrivate.h"
+#import "Foundation/NSDictionary.h"
+#import "Foundation/NSEnumerator.h"
+#import "Foundation/NSScanner.h"
+#import "Foundation/NSSet.h"
+#import "Foundation/NSDebug.h"
+#import "Foundation/NSValue.h"
+#import "GNUstepBase/GSLock.h"
+#import "GNUstepBase/GSMime.h"
 
 
 static NSMutableDictionary	*domainMap = nil;
@@ -149,7 +150,7 @@ static GSMimeParser		*mimeParser = nil;
 + (NSURLProtectionSpace*) protectionSpaceForAuthentication: (NSString*)auth
                                                 requestURL: (NSURL*)URL;
 {
-  if (auth != nil)
+  if ([auth isKindOfClass: [NSString class]] == YES)
     {
       NSString			*method = nil;
       NSURLProtectionSpace	*space;
