@@ -8,7 +8,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
    
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,7 +24,10 @@
 #ifndef _GSPrivate_h_
 #define _GSPrivate_h_
 
-#include "Foundation/NSError.h"
+#import "Foundation/NSError.h"
+
+@class	_GSInsensitiveDictionary;
+@class	_GSMutableInsensitiveDictionary;
 
 @class	NSNotification;
 
@@ -249,6 +252,46 @@ typedef enum {
 + (NSError*) _systemError: (long)number;
 @end
 
+@class  NSRunLoop;
+@class  NSLock;
+@class  NSThread;
+
+/* Used to handle events performed in one thread from another.
+ */
+@interface      GSRunLoopThreadInfo : NSObject
+{
+  @public
+  NSRunLoop             *loop;
+  NSLock                *lock;
+  NSMutableArray        *performers;
+#ifdef __MINGW32__
+  HANDLE	        event;
+#else
+  int                   inputFd;
+  int                   outputFd;
+#endif	
+}
+/* Add a performer to be run in the loop's thread.  May be called from
+ * any thread.
+ */
+- (void) addPerformer: (id)performer;
+/* Fire all pending performers in the current thread.  May only be called
+ * from the runloop when the event/descriptor is triggered.
+ */
+- (void) fire;
+/* Cancel all pending performers.
+ */
+- (void) invalidate;
+@end
+
+/* Return (and optionally create) GSRunLoopThreadInfo for the specified
+ * thread (or the current thread if aThread is nil).<br />
+ * If aThread is nil and no value is set for the current thread, create
+ * a GSRunLoopThreadInfo and set it for the current thread.
+ */
+GSRunLoopThreadInfo *
+GSRunLoopInfoForThread(NSThread *aThread) GS_ATTRIB_PRIVATE;
+
 /* Used by NSException uncaught exception handler - must not call any
  * methods/functions which might cause a recursive exception.
  */
@@ -375,6 +418,11 @@ void GSPrivateNotifyIdle(void) GS_ATTRIB_PRIVATE;
  */
 BOOL GSPrivateNotifyMore(void) GS_ATTRIB_PRIVATE;
 
+/* Function to return the current stack return addresses.
+ */
+NSMutableArray *
+GSPrivateStackAddresses(void) GS_ATTRIB_PRIVATE;
+
 /* Function to return the hash value for a small integer (used by NSNumber).
  */
 unsigned
