@@ -11,7 +11,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,10 +26,13 @@
    NSInputStream and NSOutputStream are clusters rather than concrete classes
    The inherance graph is:
    NSStream 
+   |-- GSStream
+   |   `--GSSocketStream
    |-- NSInputStream
    |   `--GSInputStream
    |      |-- GSDataInputStream
    |      |-- GSFileInputStream
+   |      |-- GSPipeInputStream (mswindows only)
    |      `-- GSSocketInputStream
    |          |-- GSInetInputStream
    |          |-- GSLocalInputStream
@@ -39,20 +42,23 @@
    |      |-- GSBufferOutputStream
    |      |-- GSDataOutputStream
    |      |-- GSFileOutputStream
+   |      |-- GSPipeOutputStream (mswindows only)
    |      `-- GSSocketOutputStream
    |          |-- GSInetOutputStream
    |          |-- GSLocalOutputStream
    |          `-- GSInet6InputStream
    `-- GSServerStream
-      `-- GSAbstractServerStream
-          `-- GSSocketServerStream
-              |-- GSInetServerStream
-              |-- GSInet6ServerStream
-              `-- GSLocalServerStream 
+       `-- GSAbstractServerStream
+           |-- GSLocalServerStream (mswindows)
+           `-- GSSocketServerStream
+               |-- GSInetServerStream
+               |-- GSInet6ServerStream
+               `-- GSLocalServerStream (gnu/linux)
 */
 
 #include <Foundation/NSStream.h>
 #include <Foundation/NSRunLoop.h>
+#include <Foundation/NSMapTable.h>
 
 /**
  * Convenience methods used to add streams to the run loop.
@@ -71,8 +77,7 @@
   BOOL                   _delegateValid;/* whether the delegate responds*/\
   NSError               *_lastError;    /* last error occured           */\
   NSStreamStatus         _currentStatus;/* current status               */\
-  NSMutableArray 	*_modes;	/* currently scheduled modes.	*/\
-  NSRunLoop 		*_runloop;	/* currently scheduled loop.	*/\
+  NSMapTable		*_loops;	/* Run loops and their modes.	*/\
   void                  *_loopID;	/* file descriptor etc.		*/\
   int			_events;	/* Signalled events.		*/\
 }
@@ -86,6 +91,10 @@
 IVARS
 @end
 
+@interface GSAbstractServerStream : GSServerStream
+IVARS
+@end
+
 @interface NSStream(Private)
 
 /**
@@ -103,6 +112,11 @@ IVARS
  */
 - (void*) _loopID;
 
+/**
+ * Place the stream in all the scheduled runloops.
+ */
+- (void) _schedule;
+
 /**
  * send an event to delegate
  */
@@ -123,11 +137,18 @@ IVARS
  * record an error based on errno
  */
 - (void) _recordError; 
+- (void) _recordError: (NSError*)anError; 
 
 /**
  * say whether there is unhandled data for the stream.
  */
 - (BOOL) _unhandledData;
+
+/**
+ * Remove the stream from all the scheduled runloops.
+ */
+- (void) _unschedule;
+
 @end
 
 @interface GSInputStream : NSInputStream
@@ -138,10 +159,6 @@ IVARS
 IVARS
 @end
 
-@interface GSAbstractServerStream : GSServerStream
-IVARS
-@end
-
 /**
  * The concrete subclass of NSInputStream that reads from the memory 
  */
@@ -151,11 +168,6 @@ IVARS
   NSData *_data;
   unsigned long _pointer;
 }
-
-/**
- * this is the bridge method for asynchronized operation. Do not call.
- */
-- (void) _dispatch;
 @end
 
 /**
@@ -168,11 +180,6 @@ IVARS
   unsigned	_capacity;
   unsigned long _pointer;
 }
-
-/**
- * this is the bridge method for asynchronized operation. Do not call.
- */
-- (void) _dispatch;
 @end
 
 /**
@@ -184,11 +191,6 @@ IVARS
   NSMutableData *_data;
   unsigned long _pointer;
 }
-
-/**
- * this is the bridge method for asynchronized operation. Do not call.
- */
-- (void) _dispatch;
 @end
 
 #endif
