@@ -8,7 +8,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,19 +21,22 @@
    Boston, MA 02111 USA.
 
    */
-#include <unistd.h>
-#include <errno.h>
 
-#include <Foundation/NSData.h>
-#include <Foundation/NSArray.h>
-#include <Foundation/NSRunLoop.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSValue.h>
-#include <Foundation/NSHost.h>
-#include <Foundation/NSDebug.h>
+#include "config.h"
 
-#include "GSStream.h"
-#include "GSPrivate.h"
+#import <Foundation/NSArray.h>
+#import <Foundation/NSByteOrder.h>
+#import <Foundation/NSData.h>
+#import <Foundation/NSDebug.h>
+#import <Foundation/NSDictionary.h>
+#import <Foundation/NSEnumerator.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSHost.h>
+#import <Foundation/NSRunLoop.h>
+#import <Foundation/NSValue.h>
+
+#import "GSStream.h"
+#import "GSPrivate.h"
 
 NSString * const NSStreamDataWrittenToMemoryStreamKey
   = @"NSStreamDataWrittenToMemoryStreamKey";
@@ -73,6 +76,15 @@ NSString * const NSStreamSOCKSProxyVersion5
 NSString * const NSStreamSOCKSProxyVersionKey
   = @"NSStreamSOCKSProxyVersionKey";
 
+NSString * const GSStreamLocalAddressKey
+  = @"GSStreamLocalAddressKey";
+NSString * const GSStreamLocalPortKey
+  = @"GSStreamLocalPortKey";
+NSString * const GSStreamRemoteAddressKey
+  = @"GSStreamRemoteAddressKey";
+NSString * const GSStreamRemotePortKey
+  = @"GSStreamRemotePortKey";
+
 
 /*
  * Determine the type of event to use when adding a stream to the run loop.
@@ -135,21 +147,9 @@ static RunLoopEventType typeForStream(NSStream *aStream)
 {
   if (_currentStatus == NSStreamStatusNotOpen)
     {
-      NSDebugMLog(@"Attempt to close unopened stream %@", self);
-    }
-  if (_currentStatus == NSStreamStatusClosed)
-    {
-      NSDebugMLog(@"Attempt to close already closed stream %@", self);
-    }
-  if (_runloop)
-    {
-      unsigned	i = [_modes count];
-
-      while (i-- > 0)
-	{
-	  [_runloop removeStream: self mode: [_modes objectAtIndex: i]];
-	}
+      NSDebugMLLog(@"NSStream", @"Attempt to close unopened stream %@", self);
     }
+  [self _unschedule];
   [self _setStatus: NSStreamStatusClosed];
   /* We don't want to send any events the the delegate after the
    * stream has been closed.
@@ -164,8 +164,11 @@ static RunLoopEventType typeForStream(NSStream *aStream)
     {
       [self close];
     }
-  DESTROY(_runloop);
-  DESTROY(_modes);
+  if (_loops != 0)
+    {
+      NSFreeMapTable(_loops);
+      _loops = 0;
+    }
   DESTROY(_properties);
   DESTROY(_lastError);
   [super dealloc];
@@ -183,7 +186,8 @@ static RunLoopEventType typeForStream(NSStream *aStream)
       _delegate = self;
       _properties = nil;
       _lastError = nil;
-      _modes = [NSMutableArray new];
+      _loops = NSCreateMapTable(NSObjectMapKeyCallBacks,
+	NSObjectMapValueCallBacks, 1);
       _currentStatus = NSStreamStatusNotOpen;
       _loopID = (void*)self;
     }
@@ -192,20 +196,13 @@ static RunLoopEventType typeForStream(NSStream *aStream)
 
 - (void) open
 {
-  if (_currentStatus != NSStreamStatusNotOpen)
+  if (_currentStatus != NSStreamStatusNotOpen
+    && _currentStatus != NSStreamStatusOpening)
     {
-      NSDebugMLog(@"Attempt to re-open stream %@", self);
+      NSDebugMLLog(@"NSStream", @"Attempt to re-open stream %@", self);
     }
   [self _setStatus: NSStreamStatusOpen];
-  if (_runloop)
-    {
-      unsigned	i = [_modes count];
-
-      while (i-- > 0)
-	{
-	  [_runloop addStream: self mode: [_modes objectAtIndex: i]];
-	}
-    }
+  [self _schedule];
   [self _sendEvent: NSStreamEventOpenCompleted];
 }
 
@@ -224,15 +221,18 @@ static RunLoopEventType typeForStream(NSStream *aStream)
 
 - (void) removeFromRunLoop: (NSRunLoop *)aRunLoop forMode: (NSString *)mode
 {
-  if (_runloop == aRunLoop)
+  if (aRunLoop != nil && mode != nil)
     {
-      if ([_modes containsObject: mode])
+      NSMutableArray	*modes;
+
+      modes = (NSMutableArray*)NSMapGet(_loops, (void*)aRunLoop);
+      if ([modes containsObject: mode])
 	{
-	  [_runloop removeStream: self mode: mode];
-	  [_modes removeObject: mode];
-	  if ([_modes count] == 0)
+	  [aRunLoop removeStream: self mode: mode];
+	  [modes removeObject: mode];
+	  if ([modes count] == 0)
 	    {
-	      DESTROY(_runloop);
+	      NSMapRemove(_loops, (void*)aRunLoop);
 	    }
 	}
     }
@@ -240,21 +240,30 @@ static RunLoopEventType typeForStream(NSStream *aStream)
 
 - (void) scheduleInRunLoop: (NSRunLoop *)aRunLoop forMode: (NSString *)mode
 {
-  NSAssert(!_runloop || _runloop == aRunLoop, 
-    @"Attempt to schedule in more than one runloop.");
-  ASSIGN(_runloop, aRunLoop);
-  if ([_modes containsObject: mode] == NO)
+  if (aRunLoop != nil && mode != nil)
     {
-      mode = [mode copy];
-      [_modes addObject: mode];
-      RELEASE(mode);
-      /* We only add open streams to the runloop .. subclasses may add
-       * streams when they are in the process of opening if they need
-       * to do so.
-       */
-      if ([self _isOpened])
+      NSMutableArray	*modes;
+
+      modes = (NSMutableArray*)NSMapGet(_loops, (void*)aRunLoop);
+      if (modes == nil)
 	{
-	  [_runloop addStream: self mode: mode];
+	  modes = [[NSMutableArray alloc] initWithCapacity: 1];
+	  NSMapInsert(_loops, (void*)aRunLoop, (void*)modes);
+	  RELEASE(modes);
+	}
+      if ([modes containsObject: mode] == NO)
+	{
+	  mode = [mode copy];
+	  [modes addObject: mode];
+	  RELEASE(mode);
+	  /* We only add open streams to the runloop .. subclasses may add
+	   * streams when they are in the process of opening if they need
+	   * to do so.
+	   */
+	  if ([self _isOpened])
+	    {
+	      [aRunLoop addStream: self mode: mode];
+	    }
 	}
     }
 }
@@ -323,6 +332,15 @@ static RunLoopEventType typeForStream(NSStream *aStream)
 {
 }
 
+- (void) _recordError: (NSError*)anError
+{
+  return;
+}
+
+- (void) _schedule
+{
+}
+
 - (void) _sendEvent: (NSStreamEvent)event
 {
 }
@@ -339,6 +357,11 @@ static RunLoopEventType typeForStream(NSStream *aStream)
 {
   return NO;
 }
+
+- (void) _unschedule
+{
+}
+
 @end
 
 @implementation	GSStream (Private)
@@ -359,17 +382,36 @@ static RunLoopEventType typeForStream(NSStream *aStream)
 {
   NSError *theError;
 
-#if	defined(__MINGW32__)
-  errno = GetLastError();
-#endif
-  theError = [NSError errorWithDomain: NSPOSIXErrorDomain
-					  code: errno
-				      userInfo: nil];
-  NSLog(@"%@ error(%d): - %@", self, errno, [NSError _last]);
-  ASSIGN(_lastError, theError);
+  theError = [NSError _last];
+  [self _recordError: theError];
+}
+
+- (void) _recordError: (NSError*)anError
+{
+  NSDebugMLLog(@"NSStream", @"record error: %@ - %@", self, anError);
+  ASSIGN(_lastError, anError);
   _currentStatus = NSStreamStatusError;
 }
 
+- (void) _schedule
+{
+  NSMapEnumerator	enumerator;
+  NSRunLoop		*k;
+  NSMutableArray	*v;
+
+  enumerator = NSEnumerateMapTable(_loops);
+  while (NSNextMapEnumeratorPair(&enumerator, (void **)(&k), (void**)&v))
+    {
+      unsigned	i = [v count];
+
+      while (i-- > 0)
+	{
+	  [k addStream: self mode: [v objectAtIndex: i]];
+	}
+    }
+  NSEndMapTableEnumeration(&enumerator);
+}
+
 - (void) _sendEvent: (NSStreamEvent)event
 {
   if (event == NSStreamEventNone)
@@ -481,6 +523,25 @@ static RunLoopEventType typeForStream(NSStream *aStream)
   return NO;
 }
 
+- (void) _unschedule
+{
+  NSMapEnumerator	enumerator;
+  NSRunLoop		*k;
+  NSMutableArray	*v;
+
+  enumerator = NSEnumerateMapTable(_loops);
+  while (NSNextMapEnumeratorPair(&enumerator, (void **)(&k), (void**)&v))
+    {
+      unsigned	i = [v count];
+
+      while (i-- > 0)
+	{
+	  [k removeStream: self mode: [v objectAtIndex: i]];
+	}
+    }
+  NSEndMapTableEnumeration(&enumerator);
+}
+
 - (BOOL) runLoopShouldBlock: (BOOL*)trigger
 {
   if (_events
@@ -492,23 +553,43 @@ static RunLoopEventType typeForStream(NSStream *aStream)
       *trigger = NO;
       return NO;
     }
-  if (_currentStatus == NSStreamStatusError &&
-    (_events & NSStreamEventErrorOccurred) == NSStreamEventErrorOccurred)
+  if (_currentStatus == NSStreamStatusError)
     {
-      /* If an error has occurred (and been handled),
-       * we should not watch for any events at all.
-       */
-      *trigger = NO;
-      return NO;
+      if ((_events & NSStreamEventErrorOccurred) == 0)
+	{
+	  /* An error has occurred but not been handled,
+	   * so we should trigger an error event at once.
+	   */
+	  *trigger = YES;
+	  return NO;
+	}
+      else
+	{
+	  /* An error has occurred (and been handled),
+	   * so we should not watch for any events at all.
+	   */
+	  *trigger = NO;
+	  return NO;
+	}
     }
-  if (_currentStatus == NSStreamStatusAtEnd &&
-    (_events & NSStreamEventEndEncountered) == NSStreamEventEndEncountered)
+  if (_currentStatus == NSStreamStatusAtEnd)
     {
-      /* If an error has occurred (and been handled),
-       * we should not watch for any events at all.
-       */
-      *trigger = NO;
-      return NO;
+      if ((_events & NSStreamEventEndEncountered) == 0)
+	{
+	  /* An end of stream has occurred but not been handled,
+	   * so we should trigger an end of stream event at once.
+	   */
+	  *trigger = YES;
+	  return NO;
+	}
+      else
+	{
+	  /* An end of stream has occurred (and been handled),
+	   * so we should not watch for any events at all.
+	   */
+	  *trigger = NO;
+	  return NO;
+	}
     }
 
   if (_loopID == (void*)self)
@@ -576,30 +657,10 @@ static RunLoopEventType typeForStream(NSStream *aStream)
     {
       return YES;
     }
-  if (_currentStatus == NSStreamStatusAtEnd)
-    {
-      if ((_events & NSStreamEventEndEncountered) == 0)
-	{
-	  /* We have not sent the appropriate event yet, so the
-           * client must not have issued a write:maxLength:
-	   * (which is the point at which we should send).
-	   */
-	  return YES;
-	}
-    }
   return NO;
 }
 
 @end
-@implementation	GSAbstractServerStream
-+ (void) initialize
-{
-  if (self == [GSAbstractServerStream class])
-    {
-      GSObjCAddClassBehavior(self, [GSStream class]);
-    }
-}
-@end
 
 
 @implementation GSDataInputStream
@@ -666,7 +727,6 @@ static RunLoopEventType typeForStream(NSStream *aStream)
   else
     {
       [self _setStatus: NSStreamStatusAtEnd];
-      [self _sendEvent: NSStreamEventEndEncountered];
     }
   return copySize;
 }
@@ -748,7 +808,6 @@ static RunLoopEventType typeForStream(NSStream *aStream)
       if (len == 0)
 	{
 	  [self _setStatus: NSStreamStatusAtEnd];
-	  [self _sendEvent: NSStreamEventEndEncountered];
 	  return 0;
 	}
     }
@@ -850,403 +909,65 @@ static RunLoopEventType typeForStream(NSStream *aStream)
 @end
 
 
-/*
- * States for socks connection negotiation
- */
-static NSString * const GSSOCKSOfferAuth = @"GSSOCKSOfferAuth";
-static NSString * const GSSOCKSRecvAuth = @"GSSOCKSRecvAuth";
-static NSString * const GSSOCKSSendAuth = @"GSSOCKSSendAuth";
-static NSString * const GSSOCKSAckAuth = @"GSSOCKSAckAuth";
-static NSString * const GSSOCKSSendConn = @"GSSOCKSSendConn";
-static NSString * const GSSOCKSAckConn = @"GSSOCKSAckConn";
+@class  GSInetInputStream;
+@class  GSInet6InputStream;
+@class  GSInetOutputStream;
+@class  GSInet6OutputStream;
+@class  GSInetServerStream;
+@class  GSInet6ServerStream;
+@class  GSLocalServerStream;
 
-@interface	GSSOCKS
-{
-  NSString		*state;
-  NSString		*addr;
-  int			port;
-  int			roffset;
-  int			woffset;
-  int			rwant;
-  unsigned char		rbuffer[128];
-  NSInputStream		*istream;
-  NSOutputStream	*ostream;
-}
-- (NSString*) addr;
-- (id) initToAddr: (NSString*)_addr port: (int)_port;
-- (int) port;
-- (NSString*) stream: (NSStream*)stream SOCKSEvent: (NSStreamEvent)event;
-@end
+@implementation GSServerStream
 
-@implementation	GSSOCKS
-- (NSString*) addr
++ (id) serverStreamToAddr: (NSString*)addr port: (int)port
 {
-  return addr;
+  GSServerStream *s;
+
+  // try inet first, then inet6
+  s = [[GSInetServerStream alloc] initToAddr: addr port: port];
+  if (!s)
+    s = [[GSInet6ServerStream alloc] initToAddr: addr port: port];
+  return AUTORELEASE(s);
 }
 
-- (id) initToAddr: (NSString*)_addr port: (int)_port
++ (id) serverStreamToAddr: (NSString*)addr
 {
-  ASSIGNCOPY(addr, _addr);
-  port = _port;
-  state = GSSOCKSOfferAuth;
+  return AUTORELEASE([[GSLocalServerStream alloc] initToAddr: addr]);
+}
+
+- (id) initToAddr: (NSString*)addr port: (int)port
+{
+  RELEASE(self);
+  // try inet first, then inet6
+  self = [[GSInetServerStream alloc] initToAddr: addr port: port];
+  if (!self)
+    self = [[GSInet6ServerStream alloc] initToAddr: addr port: port];
   return self;
 }
 
-- (int) port
+- (id) initToAddr: (NSString*)addr
 {
-  return port;
+  RELEASE(self);
+  return [[GSLocalServerStream alloc] initToAddr: addr];
 }
 
-- (NSString*) stream: (NSStream*)stream SOCKSEvent: (NSStreamEvent)event
+- (void) acceptWithInputStream: (NSInputStream **)inputStream 
+                  outputStream: (NSOutputStream **)outputStream
 {
-  NSString		*error = nil;
-  NSDictionary		*conf;
-  NSString		*user;
-  NSString		*pass;
-
-  if (event == NSStreamEventErrorOccurred
-    || [stream streamStatus] == NSStreamStatusError
-    || [stream streamStatus] == NSStreamStatusClosed)
-    {
-      return @"SOCKS errur during negotiation";
-    }
-
-  conf = [stream propertyForKey: NSStreamSOCKSProxyConfigurationKey];
-  user = [conf objectForKey: NSStreamSOCKSProxyUserKey];
-  pass = [conf objectForKey: NSStreamSOCKSProxyPasswordKey];
-  if ([[conf objectForKey: NSStreamSOCKSProxyVersionKey]
-    isEqual: NSStreamSOCKSProxyVersion4] == YES)
-    {
-    }
-  else
-    {
-      again:
-
-      if (state == GSSOCKSOfferAuth)
-	{
-	  int		result;
-	  int		want;
-	  unsigned char	buf[4];
-
-	  /*
-	   * Authorisation record is at least three bytes -
-	   *   socks version (5)
-	   *   authorisation method bytes to follow (1)
-	   *   say we do no authorisation (0)
-	   *   say we do user/pass authorisation (2)
-	   */
-	  buf[0] = 5;
-	  if (user && pass)
-	    {
-	      buf[1] = 2;
-	      buf[2] = 2;
-	      buf[3] = 0;
-	      want = 4;
-	    }
-	  else
-	    {
-	      buf[1] = 1;
-	      buf[2] = 0;
-	      want = 3;
-	    }
-
-	  result = [ostream write: buf + woffset maxLength: 4 - woffset];
-	  if (result == 0)
-	    {
-	      error = @"end-of-file during SOCKS negotiation";
-	    }
-	  else if (result > 0)
-	    {
-	      woffset += result;
-	      if (woffset == want)
-		{
-		  woffset = 0;
-		  state = GSSOCKSRecvAuth;
-		  goto again;
-		}
-	    }
-	}
-      else if (state == GSSOCKSRecvAuth)
-	{
-	  int	result;
-
-	  result = [istream read: rbuffer + roffset maxLength: 2 - roffset];
-	  if (result == 0)
-	    {
-	      error = @"SOCKS end-of-file during negotiation";
-	    }
-	  else if (result > 0)
-	    {
-	      roffset += result;
-	      if (roffset == 2)
-		{
-		  roffset = 0;
-		  if (rbuffer[0] != 5)
-		    {
-		      error = @"SOCKS authorisation response had wrong version";
-		    }
-		  else if (rbuffer[1] == 0)
-		    {
-		      state = GSSOCKSSendConn;
-		      goto again;
-		    }
-		  else if (rbuffer[1] == 2)
-		    {
-		      state = GSSOCKSSendAuth;
-		      goto again;
-		    }
-		  else
-		    {
-		      error = @"SOCKS authorisation response had wrong method";
-		    }
-		}
-	    }
-	}
-      else if (state == GSSOCKSSendAuth)
-	{
-	  NSData	*u = [user dataUsingEncoding: NSUTF8StringEncoding];
-	  unsigned	ul = [u length];
-	  NSData	*p = [pass dataUsingEncoding: NSUTF8StringEncoding];
-	  unsigned	pl = [p length];
-
-	  if (ul < 1 || ul > 255)
-	    {
-	      error = @"NSStreamSOCKSProxyUserKey value too long";
-	    }
-	  else if (ul < 1 || ul > 255)
-	    {
-	      error = @"NSStreamSOCKSProxyPasswordKey value too long";
-	    }
-	  else
-	    {
-	      int		want = ul + pl + 3;
-	      unsigned char	buf[want];
-	      int		result;
-
-	      buf[0] = 5;
-	      buf[1] = ul;
-	      memcpy(buf + 2, [u bytes], ul);
-	      buf[ul + 2] = pl;
-	      memcpy(buf + ul + 3, [p bytes], pl);
-	      result = [ostream write: buf + woffset maxLength: want - woffset];
-	      if (result == 0)
-		{
-		  error = @"SOCKS end-of-file during negotiation";
-		}
-	      else if (result > 0)
-		{
-		  woffset += result;
-		  if (woffset == want)
-		    {
-		      state = GSSOCKSAckAuth;
-		      goto again;
-		    }
-		}
-	    }
-	}
-      else if (state == GSSOCKSAckAuth)
-	{
-	  int	result;
-
-	  result = [istream read: rbuffer + roffset maxLength: 2 - roffset];
-	  if (result == 0)
-	    {
-	      error = @"SOCKS end-of-file during negotiation";
-	    }
-	  else if (result > 0)
-	    {
-	      roffset += result;
-	      if (roffset == 2)
-		{
-		  roffset = 0;
-		  if (rbuffer[0] != 5)
-		    {
-		      error = @"SOCKS authorisation response had wrong version";
-		    }
-		  else if (rbuffer[1] == 0)
-		    {
-		      state = GSSOCKSSendConn;
-		      goto again;
-		    }
-		  else if (rbuffer[1] == 2)
-		    {
-		      error = @"SOCKS authorisation failed";
-		    }
-		}
-	    }
-	}
-      else if (state == GSSOCKSSendConn)
-	{
-	  unsigned char	buf[10];
-	  int		want = 10;
-	  int		result;
-	  const char	*ptr;
-
-	  /*
-	   * Connect command is ten bytes -
-	   *   socks version
-	   *   connect command
-	   *   reserved byte
-	   *   address type
-	   *   address 4 bytes (big endian)
-	   *   port 2 bytes (big endian)
-	   */
-	  buf[0] = 5;	// Socks version number
-	  buf[1] = 1;	// Connect command
-	  buf[2] = 0;	// Reserved
-	  buf[3] = 1;	// Address type (IPV4)
-	  ptr = [addr lossyCString];
-	  buf[4] = atoi(ptr);
-	  while (isdigit(*ptr))
-	    ptr++;
-	  ptr++;
-	  buf[5] = atoi(ptr);
-	  while (isdigit(*ptr))
-	    ptr++;
-	  ptr++;
-	  buf[6] = atoi(ptr);
-	  while (isdigit(*ptr))
-	    ptr++;
-	  ptr++;
-	  buf[7] = atoi(ptr);
-	  buf[8] = ((port & 0xff00) >> 8);
-	  buf[9] = (port & 0xff);
-
-	  result = [ostream write: buf + woffset maxLength: want - woffset];
-	  if (result == 0)
-	    {
-	      error = @"SOCKS end-of-file during negotiation";
-	    }
-	  else if (result > 0)
-	    {
-	      woffset += result;
-	      if (woffset == want)
-		{
-		  rwant = 5;
-		  state = GSSOCKSAckConn;
-		  goto again;
-		}
-	    }
-	}
-      else if (state == GSSOCKSAckConn)
-	{
-	  int	result;
-
-	  result = [istream read: rbuffer + roffset maxLength: rwant - roffset];
-	  if (result == 0)
-	    {
-	      error = @"SOCKS end-of-file during negotiation";
-	    }
-	  else if (result > 0)
-	    {
-	      roffset += result;
-	      if (roffset == rwant)
-		{
-		  if (rbuffer[0] != 5)
-		    {
-		      error = @"connect response from SOCKS had wrong version";
-		    }
-		  else if (rbuffer[1] != 0)
-		    {
-		      switch (rbuffer[1])
-			{
-			  case 1:
-			    error = @"SOCKS server general failure";
-			    break;
-			  case 2:
-			    error = @"SOCKS server says permission denied";
-			    break;
-			  case 3:
-			    error = @"SOCKS server says network unreachable";
-			    break;
-			  case 4:
-			    error = @"SOCKS server says host unreachable";
-			    break;
-			  case 5:
-			    error = @"SOCKS server says connection refused";
-			    break;
-			  case 6:
-			    error = @"SOCKS server says connection timed out";
-			    break;
-			  case 7:
-			    error = @"SOCKS server says command not supported";
-			    break;
-			  case 8:
-			    error = @"SOCKS server says address not supported";
-			    break;
-			  default:
-			    error = @"connect response from SOCKS was failure";
-			    break;
-			}
-		    }
-		  else if (rbuffer[3] == 1)
-		    {
-		      rwant = 10;		// Fixed size (IPV4) address
-		    }
-		  else if (rbuffer[3] == 3)
-		    {
-		      rwant = 7 + rbuffer[4];	// Domain name leading length
-		    }
-		  else if (rbuffer[3] == 4)
-		    {
-		      rwant = 22;		// Fixed size (IPV6) address
-		    }
-		  else
-		    {
-		      error = @"SOCKS server returned unknown address type";
-		    }
-		  if (error == nil)
-		    {
-		      if (roffset < rwant)
-			{
-			  goto again;	// Need address/port bytes
-			}
-		      else
-			{
-			  NSString	*a;
-
-			  error = @"";	// success
-			  if (rbuffer[3] == 1)
-			    {
-			      a = [NSString stringWithFormat: @"%d.%d.%d.%d",
-			        rbuffer[4], rbuffer[5], rbuffer[6], rbuffer[7]];
-			    }
-			  else if (rbuffer[3] == 3)
-			    {
-			      rbuffer[rwant] = '\0';
-			      a = [NSString stringWithUTF8String:
-			        (const char*)rbuffer];
-			    }
-			  else
-			    {
-			      unsigned char	buf[40];
-			      int		i = 4;
-			      int		j = 0;
-
-			      while (i < rwant)
-			        {
-				  int	val = rbuffer[i++] * 256 + rbuffer[i++];
-
-				  if (i > 4)
-				    {
-				      buf[j++] = ':';
-				    }
-				  sprintf((char*)&buf[j], "%04x", val);
-				  j += 4;
-				}
-			      a = [NSString stringWithUTF8String:
-			        (const char*)buf];
-			    }
-			  ASSIGN(addr, a);
-			  port =  rbuffer[rwant-1] * 256 * rbuffer[rwant-2];
-			}
-		    }
-		}
-	    }
-	}
-    }
-
-  return error;
+  [self subclassResponsibility: _cmd];
 }
 
 @end
+
+@implementation GSAbstractServerStream
+
++ (void) initialize
+{
+  if (self == [GSAbstractServerStream class])
+    {
+      GSObjCAddClassBehavior(self, [GSStream class]);
+    }
+}
+
+@end
+
