@@ -8,7 +8,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
    
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,30 +27,31 @@
 /*
  * Headers needed by many URL loading classes
  */
-#include "config.h"
-#include "GNUstepBase/preface.h"
-#include "Foundation/NSArray.h"
-#include "Foundation/NSAutoreleasePool.h"
-#include "Foundation/NSData.h"
-#include "Foundation/NSDictionary.h"
-#include "Foundation/NSException.h"
-#include "Foundation/NSHTTPCookie.h"
-#include "Foundation/NSHTTPCookieStorage.h"
-#include "Foundation/NSLock.h"
-#include "Foundation/NSStream.h"
-#include "Foundation/NSString.h"
-#include "Foundation/NSURL.h"
-#include "Foundation/NSURLAuthenticationChallenge.h"
-#include "Foundation/NSURLCache.h"
-#include "Foundation/NSURLConnection.h"
-#include "Foundation/NSURLCredential.h"
-#include "Foundation/NSURLCredentialStorage.h"
-#include "Foundation/NSURLDownload.h"
-#include "Foundation/NSURLError.h"
-#include "Foundation/NSURLProtectionSpace.h"
-#include "Foundation/NSURLProtocol.h"
-#include "Foundation/NSURLRequest.h"
-#include "Foundation/NSURLResponse.h"
+#import "config.h"
+#import "GNUstepBase/preface.h"
+#import "Foundation/NSArray.h"
+#import "Foundation/NSAutoreleasePool.h"
+#import "Foundation/NSData.h"
+#import "Foundation/NSDictionary.h"
+#import "Foundation/NSEnumerator.h"
+#import "Foundation/NSException.h"
+#import "Foundation/NSHTTPCookie.h"
+#import "Foundation/NSHTTPCookieStorage.h"
+#import "Foundation/NSLock.h"
+#import "Foundation/NSStream.h"
+#import "Foundation/NSString.h"
+#import "Foundation/NSURL.h"
+#import "Foundation/NSURLAuthenticationChallenge.h"
+#import "Foundation/NSURLCache.h"
+#import "Foundation/NSURLConnection.h"
+#import "Foundation/NSURLCredential.h"
+#import "Foundation/NSURLCredentialStorage.h"
+#import "Foundation/NSURLDownload.h"
+#import "Foundation/NSURLError.h"
+#import "Foundation/NSURLProtectionSpace.h"
+#import "Foundation/NSURLProtocol.h"
+#import "Foundation/NSURLRequest.h"
+#import "Foundation/NSURLResponse.h"
 
 /*
  * Private accessors for URL loading classes
@@ -62,6 +63,13 @@
 @end
 
 
+@interface	NSURLResponse (Private)
+- (void) _setHeaders: (id)headers;
+- (void) _setStatusCode: (int)code text: (NSString*)text;
+- (void) _setValue: (NSString *)value forHTTPHeaderField: (NSString *)field;
+- (NSString*) _valueForHTTPHeaderField: (NSString*)field;
+@end
+
 
 
 /*
