@@ -14,7 +14,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,30 +27,33 @@
    Boston, MA 02111 USA.
 
    <title>NSArray class reference</title>
-   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
+   $Date: 2008-06-12 11:44:00 +0100 (Thu, 12 Jun 2008) $ $Revision: 26630 $
    */
 
-#include "config.h"
-#include "Foundation/NSArray.h"
-#include "Foundation/NSCoder.h"
-#include "Foundation/NSData.h"
-#include "Foundation/NSString.h"
-#include "Foundation/NSRange.h"
+#import "config.h"
+#import "Foundation/NSArray.h"
+#import "Foundation/NSEnumerator.h"
+#import "Foundation/NSCoder.h"
+#import "Foundation/NSData.h"
+#import "Foundation/NSString.h"
+#import "Foundation/NSRange.h"
 #include <limits.h>
-#include "Foundation/NSUtilities.h"
-#include "Foundation/NSException.h"
-#include "Foundation/NSAutoreleasePool.h"
-#include "Foundation/NSThread.h"
-#include "Foundation/NSMapTable.h"
-#include "Foundation/NSLock.h"
-#include "Foundation/NSDebug.h"
-#include "Foundation/NSValue.h"
-#include "Foundation/NSNull.h"
-#include "Foundation/NSUserDefaults.h"
+#import "Foundation/NSException.h"
+#import "Foundation/NSAutoreleasePool.h"
+#import "Foundation/NSThread.h"
+#import "Foundation/NSMapTable.h"
+#import "Foundation/NSLock.h"
+#import "Foundation/NSDebug.h"
+#import "Foundation/NSValue.h"
+#import "Foundation/NSNull.h"
+#import "Foundation/NSKeyValueCoding.h"
+#import "Foundation/NSSet.h"
+#import "Foundation/NSUserDefaults.h"
+#import "Foundation/NSIndexSet.h"
 // For private method _decodeArrayOfObjectsForKey:
-#include "Foundation/NSKeyedArchiver.h"
-#include "GNUstepBase/GSCategories.h"
-#include "GSPrivate.h"
+#import "Foundation/NSKeyedArchiver.h"
+#import "GNUstepBase/GSCategories.h"
+#import "GSPrivate.h"
 
 static BOOL GSMacOSXCompatiblePropertyLists(void)
 {
@@ -835,6 +838,21 @@ static SEL	rlSel;
   return nil;
 }
 
+- (NSArray *) objectsAtIndexes: (NSIndexSet *)indexes
+{
+  //FIXME: probably slow!
+  NSMutableArray *group = [NSMutableArray arrayWithCapacity: [indexes count]];
+
+  unsigned i = [indexes firstIndex];
+  while (i != NSNotFound)
+    {
+      [group addObject: [self objectAtIndex: i]];
+      i = [indexes indexGreaterThanIndex: i];
+    }
+
+  return [group copy];
+}
+
 - (BOOL) isEqual: (id)anObject
 {
   if (self == anObject)
@@ -844,6 +862,7 @@ static SEL	rlSel;
   return NO;
 }
 
+
 /**
  * Returns YES if the receiver is equal to otherArray, NO otherwise.
  */
@@ -1296,6 +1315,238 @@ compare(id elem1, id elem2, void* context)
   return result;
 }
 
+- (id) valueForKeyPath: (NSString*)path
+{
+  id result;
+
+  if ([path hasPrefix: @"@"])
+    {
+      NSRange   r;
+
+      r = [path rangeOfString: @"."];
+      if (r.length == 0)
+        {
+          if ([path isEqualToString: @"@count"] == YES)
+            {
+              result = [NSNumber numberWithUnsignedInt: [self count]];
+            }
+          else
+            {
+              result = [self valueForKey: path];
+            }
+        }
+      else
+        {
+          NSString      *op = [path substringToIndex: r.location];
+          NSString      *rem = [path substringFromIndex: NSMaxRange(r)];
+          unsigned      count = [self count];
+
+          if ([op isEqualToString: @"@count"] == YES)
+            {
+              result = [NSNumber numberWithUnsignedInt: count];
+            }
+          else if ([op isEqualToString: @"@avg"] == YES)
+            {
+              double        d = 0;
+
+              if (count > 0)
+                {
+                  NSEnumerator  *e = [self objectEnumerator];
+                  id            o;
+                  
+                  while ((o = [e nextObject]) != nil)
+                    {
+                      d += [[o valueForKeyPath: rem] doubleValue];
+                    }
+                  d /= count;
+                }
+              result = [NSNumber numberWithDouble: d];
+            }
+          else if ([op isEqualToString: @"@max"] == YES)
+            {
+              if (count > 0)
+                {
+                  NSEnumerator  *e = [self objectEnumerator];
+                  id            o;
+                  
+                  while ((o = [e nextObject]) != nil)
+                    {
+                      o = [o valueForKeyPath: rem];
+                      if (result == nil
+                        || [result compare: o] == NSOrderedAscending)
+                        {
+                          result = o;
+                        }
+                    }
+                }
+            }
+          else if ([op isEqualToString: @"@min"] == YES)
+            {
+              if (count > 0)
+                {
+                  NSEnumerator  *e = [self objectEnumerator];
+                  id            o;
+                  
+                  while ((o = [e nextObject]) != nil)
+                    {
+                      o = [o valueForKeyPath: rem];
+                      if (result == nil
+                        || [result compare: o] == NSOrderedDescending)
+                        {
+                          result = o;
+                        }
+                    }
+                }
+            }
+          else if ([op isEqualToString: @"@sum"] == YES)
+            {
+              double        d = 0;
+
+              if (count > 0)
+                {
+                  NSEnumerator  *e = [self objectEnumerator];
+                  id            o;
+                  
+                  while ((o = [e nextObject]) != nil)
+                    {
+                      d += [[o valueForKeyPath: rem] doubleValue];
+                    }
+                }
+              result = [NSNumber numberWithDouble: d];
+            }
+          else if ([op isEqualToString: @"@distinctUnionOfArrays"] == YES)
+            {
+              if (count > 0)
+                {
+                  NSEnumerator  *e = [self objectEnumerator];
+                  id            o;
+                  
+                  result = [NSMutableSet set];
+                  while ((o = [e nextObject]) != nil)
+                    {
+                      o = [o valueForKeyPath: rem];
+                      [result addObjectsFromArray: o];
+                    }
+                  result = [result allObjects];
+                }
+              else
+                {
+                  result = [NSArray array];
+                }
+            }
+          else if ([op isEqualToString: @"@distinctUnionOfObjects"] == YES)
+            {
+              if (count > 0)
+                {
+                  NSEnumerator  *e = [self objectEnumerator];
+                  id            o;
+                  
+                  result = [NSMutableSet set];
+                  while ((o = [e nextObject]) != nil)
+                    {
+                      o = [o valueForKeyPath: rem];
+                      [result addObject: o];
+                    }
+                  result = [result allObjects];
+                }
+              else
+                {
+                  result = [NSArray array];
+                }
+            }
+          else if ([op isEqualToString: @"@distinctUnionOfSets"] == YES)
+            {
+              if (count > 0)
+                {
+                  NSEnumerator  *e = [self objectEnumerator];
+                  id            o;
+                  
+                  result = [NSMutableSet set];
+                  while ((o = [e nextObject]) != nil)
+                    {
+                      o = [o valueForKeyPath: rem];
+                      [result addObjectsFromArray: [o allObjects]];
+                    }
+                  result = [result allObjects];
+                }
+              else
+                {
+                  result = [NSArray array];
+                }
+            }
+          else if ([op isEqualToString: @"@unionOfArrays"] == YES)
+            {
+              if (count > 0)
+                {
+                  NSEnumerator  *e = [self objectEnumerator];
+                  id            o;
+                  
+                  result = [GSMutableArray array];
+                  while ((o = [e nextObject]) != nil)
+                    {
+                      o = [o valueForKeyPath: rem];
+                      [result addObjectsFromArray: o];
+                    }
+                  [result makeImmutableCopyOnFail: NO];
+                }
+              else
+                {
+                  result = [NSArray array];
+                }
+            }
+          else if ([op isEqualToString: @"@unionOfObjects"] == YES)
+            {
+              if (count > 0)
+                {
+                  NSEnumerator  *e = [self objectEnumerator];
+                  id            o;
+                  
+                  result = [GSMutableArray array];
+                  while ((o = [e nextObject]) != nil)
+                    {
+                      o = [o valueForKeyPath: rem];
+                      [result addObject: o];
+                    }
+                  [result makeImmutableCopyOnFail: NO];
+                }
+              else
+                {
+                  result = [NSArray array];
+                }
+            }
+          else if ([op isEqualToString: @"@unionOfSets"] == YES)
+            {
+              if (count > 0)
+                {
+                  NSEnumerator  *e = [self objectEnumerator];
+                  id            o;
+                  
+                  result = [GSMutableArray array];
+                  while ((o = [e nextObject]) != nil)
+                    {
+                      o = [o valueForKeyPath: rem];
+                      [result addObjectsFromArray: [o allObjects]];
+                    }
+                  [result makeImmutableCopyOnFail: NO];
+                }
+              else
+                {
+                  result = [NSArray array];
+                }
+            }
+          else
+            {
+              result = [super valueForKeyPath: path];
+            }
+        }
+    }
+  else
+    {
+      result = [super valueForKeyPath: path];
+    }
+
+  return result;
+}
 @end
 
 
@@ -1384,6 +1635,13 @@ compare(id elem1, id elem2, void* context)
   [self subclassResponsibility: _cmd];
 }
 
+/** Not implemented
+ */
+- (void) replaceObjectsAtIndexes: (NSIndexSet *)indexes
+                     withObjects: (NSArray *)objects
+{
+}
+
 /**
  * Replaces objects in the receiver with those from anArray.<br />
  * Raises an exception if given a range extending beyond the array.<br />
@@ -1425,6 +1683,12 @@ compare(id elem1, id elem2, void* context)
   [self subclassResponsibility: _cmd];
 }
 
+/** Not implemented
+ */
+- (void) insertObjects: (NSArray *)objects atIndexes: (NSIndexSet *)indexes
+{
+}
+
 /** <override-subclass />
  * Removes an object from the receiver at the specified location.<br />
  * The size of the array decreases by one.<br />
@@ -1706,6 +1970,22 @@ compare(id elem1, id elem2, void* context)
   [self addObjectsFromArray: otherArray];
 }
 
+/**
+ * Removes objects from the receiver at the indices supplied by an NSIndexSet
+ */
+- (void) removeObjectsAtIndexes: (NSIndexSet *)indexes
+{
+  unsigned count = [indexes count];
+  unsigned indexArray[count];
+
+  [indexes getIndexes: indexArray 
+             maxCount: count 
+         inIndexRange: NULL];
+
+  [self removeObjectsFromIndices: indexArray
+                      numIndices: count];
+}
+
 /**
  * Supplied with a C array of indices containing count values, this method
  * removes all corresponding objects from the receiver.  The objects are
