@@ -11,7 +11,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,11 +24,12 @@
    Boston, MA 02111 USA.
 
    <title>NSDebug utilities reference</title>
-   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
+   $Date: 2008-06-12 11:44:00 +0100 (Thu, 12 Jun 2008) $ $Revision: 26630 $
    */
 
 #include "config.h"
 #include <stdio.h>
+#include "GSPrivate.h"
 #include "GNUstepBase/GSLock.h"
 #include "Foundation/NSArray.h"
 #include "Foundation/NSData.h"
@@ -38,6 +39,7 @@
 #include "Foundation/NSNotification.h"
 #include "Foundation/NSNotificationQueue.h"
 #include "Foundation/NSThread.h"
+#include "Foundation/NSValue.h"
 
 typedef struct {
   Class	class;
@@ -854,17 +856,10 @@ GSDebugMethodMsg(id obj, SEL sel, const char *file, int line, NSString *fmt)
   return message;
 }
 
-unsigned NSCountFrames(void)
-{
-   unsigned    x = 0;
-
-   while (NSFrameAddress(x + 1)) x++;
-
-   return x;
-}
-
-#define _NS_FRAME_HACK(a) case a: val = __builtin_frame_address(a + 1); break;
-#define _NS_RETURN_HACK(a) case a: val = __builtin_return_address(a + 1); break;
+#define _NS_FRAME_HACK(a) \
+case a: env->addr = __builtin_frame_address(a + 1); break;
+#define _NS_RETURN_HACK(a) \
+case a: env->addr = __builtin_return_address(a + 1); break;
 
 /*
  * The following horrible signal handling code is a workaround for the fact
@@ -882,9 +877,17 @@ unsigned NSCountFrames(void)
 
 #if	defined(__MINGW32__)
 #include <setjmp.h>
+#define SIGBUS  SIGILL
 #endif
 
-static jmp_buf *
+typedef struct {
+  jmp_buf       buf;
+  void          *addr;
+  void          (*bus)(int);
+  void          (*segv)(int);
+} jbuf_type;
+
+static jbuf_type *
 jbuf()
 {
   NSMutableData	*d;
@@ -892,35 +895,30 @@ jbuf()
   d = [[[NSThread currentThread] threadDictionary] objectForKey: @"GSjbuf"];
   if (d == nil)
     {
-      d = [[NSMutableData alloc] initWithLength:
-	sizeof(jmp_buf) + sizeof(void(*)(int))];
+      d = [[NSMutableData alloc] initWithLength: sizeof(jbuf_type)];
       [[[NSThread currentThread] threadDictionary] setObject: d
 						      forKey: @"GSjbuf"];
       RELEASE(d);
     }
-  return (jmp_buf*)[d mutableBytes];
+  return (jbuf_type*)[d mutableBytes];
 }
 
 static void
 recover(int sig)
 {
-  jmp_buf	*env = jbuf();
-
-  longjmp(*env, 1);
+  longjmp(jbuf()->buf, 1);
 }
 
 void *
 NSFrameAddress(int offset)
 {
-  jmp_buf	*env;
-  void		(*old)(int);
-  void		*val;
+  jbuf_type     *env;
 
   env = jbuf();
-  if (setjmp(*env) == 0)
+  if (setjmp(env->buf) == 0)
     {
-      old = signal(SIGSEGV, recover);
-      memcpy(env + 1, &old, sizeof(old));
+      env->segv = signal(SIGSEGV, recover);
+      env->bus = signal(SIGBUS, recover);
       switch (offset)
 	{
 	  _NS_FRAME_HACK(0); _NS_FRAME_HACK(1); _NS_FRAME_HACK(2);
@@ -957,32 +955,93 @@ NSFrameAddress(int offset)
 	  _NS_FRAME_HACK(93); _NS_FRAME_HACK(94); _NS_FRAME_HACK(95);
 	  _NS_FRAME_HACK(96); _NS_FRAME_HACK(97); _NS_FRAME_HACK(98);
 	  _NS_FRAME_HACK(99);
-	  default: val = NULL; break;
+	  default: env->addr = NULL; break;
 	}
-      signal(SIGSEGV, old);
+      signal(SIGSEGV, env->segv);
+      signal(SIGBUS, env->bus);
+    }
+  else
+    {
+      signal(SIGSEGV, env->segv);
+      signal(SIGBUS, env->bus);
+      env->addr = NULL;
+    }
+  return env->addr;
+}
+
+unsigned NSCountFrames(void)
+{
+  jbuf_type	*env;
+
+  env = jbuf();
+  if (setjmp(env->buf) == 0)
+    {
+      env->segv = signal(SIGSEGV, recover);
+      env->bus = signal(SIGBUS, recover);
+      env->addr = 0;
+
+#define _NS_COUNT_HACK(X) if (__builtin_frame_address(X + 1) == 0) \
+        goto done; else env->addr = (void*)(X + 1);
+
+      _NS_COUNT_HACK(0); _NS_COUNT_HACK(1); _NS_COUNT_HACK(2);
+      _NS_COUNT_HACK(3); _NS_COUNT_HACK(4); _NS_COUNT_HACK(5);
+      _NS_COUNT_HACK(6); _NS_COUNT_HACK(7); _NS_COUNT_HACK(8);
+      _NS_COUNT_HACK(9); _NS_COUNT_HACK(10); _NS_COUNT_HACK(11);
+      _NS_COUNT_HACK(12); _NS_COUNT_HACK(13); _NS_COUNT_HACK(14);
+      _NS_COUNT_HACK(15); _NS_COUNT_HACK(16); _NS_COUNT_HACK(17);
+      _NS_COUNT_HACK(18); _NS_COUNT_HACK(19); _NS_COUNT_HACK(20);
+      _NS_COUNT_HACK(21); _NS_COUNT_HACK(22); _NS_COUNT_HACK(23);
+      _NS_COUNT_HACK(24); _NS_COUNT_HACK(25); _NS_COUNT_HACK(26);
+      _NS_COUNT_HACK(27); _NS_COUNT_HACK(28); _NS_COUNT_HACK(29);
+      _NS_COUNT_HACK(30); _NS_COUNT_HACK(31); _NS_COUNT_HACK(32);
+      _NS_COUNT_HACK(33); _NS_COUNT_HACK(34); _NS_COUNT_HACK(35);
+      _NS_COUNT_HACK(36); _NS_COUNT_HACK(37); _NS_COUNT_HACK(38);
+      _NS_COUNT_HACK(39); _NS_COUNT_HACK(40); _NS_COUNT_HACK(41);
+      _NS_COUNT_HACK(42); _NS_COUNT_HACK(43); _NS_COUNT_HACK(44);
+      _NS_COUNT_HACK(45); _NS_COUNT_HACK(46); _NS_COUNT_HACK(47);
+      _NS_COUNT_HACK(48); _NS_COUNT_HACK(49); _NS_COUNT_HACK(50);
+      _NS_COUNT_HACK(51); _NS_COUNT_HACK(52); _NS_COUNT_HACK(53);
+      _NS_COUNT_HACK(54); _NS_COUNT_HACK(55); _NS_COUNT_HACK(56);
+      _NS_COUNT_HACK(57); _NS_COUNT_HACK(58); _NS_COUNT_HACK(59);
+      _NS_COUNT_HACK(60); _NS_COUNT_HACK(61); _NS_COUNT_HACK(62);
+      _NS_COUNT_HACK(63); _NS_COUNT_HACK(64); _NS_COUNT_HACK(65);
+      _NS_COUNT_HACK(66); _NS_COUNT_HACK(67); _NS_COUNT_HACK(68);
+      _NS_COUNT_HACK(69); _NS_COUNT_HACK(70); _NS_COUNT_HACK(71);
+      _NS_COUNT_HACK(72); _NS_COUNT_HACK(73); _NS_COUNT_HACK(74);
+      _NS_COUNT_HACK(75); _NS_COUNT_HACK(76); _NS_COUNT_HACK(77);
+      _NS_COUNT_HACK(78); _NS_COUNT_HACK(79); _NS_COUNT_HACK(80);
+      _NS_COUNT_HACK(81); _NS_COUNT_HACK(82); _NS_COUNT_HACK(83);
+      _NS_COUNT_HACK(84); _NS_COUNT_HACK(85); _NS_COUNT_HACK(86);
+      _NS_COUNT_HACK(87); _NS_COUNT_HACK(88); _NS_COUNT_HACK(89);
+      _NS_COUNT_HACK(90); _NS_COUNT_HACK(91); _NS_COUNT_HACK(92);
+      _NS_COUNT_HACK(93); _NS_COUNT_HACK(94); _NS_COUNT_HACK(95);
+      _NS_COUNT_HACK(96); _NS_COUNT_HACK(97); _NS_COUNT_HACK(98);
+      _NS_COUNT_HACK(99);
+
+done:
+      signal(SIGSEGV, env->segv);
+      signal(SIGBUS, env->bus);
     }
   else
     {
       env = jbuf();
-      memcpy(&old, env + 1, sizeof(old));
-      signal(SIGSEGV, old);
-      val = NULL;
+      signal(SIGSEGV, env->segv);
+      signal(SIGBUS, env->bus);
     }
-  return val;
+
+  return (unsigned)(uintptr_t)env->addr;
 }
 
 void *
 NSReturnAddress(int offset)
 {
-  jmp_buf	*env;
-  void		(*old)(int);
-  void		*val;
+  jbuf_type	*env;
 
   env = jbuf();
-  if (setjmp(*env) == 0)
+  if (setjmp(env->buf) == 0)
     {
-      old = signal(SIGSEGV, recover);
-      memcpy(env + 1, &old, sizeof(old));
+      env->segv = signal(SIGSEGV, recover);
+      env->bus = signal(SIGBUS, recover);
       switch (offset)
 	{
 	  _NS_RETURN_HACK(0); _NS_RETURN_HACK(1); _NS_RETURN_HACK(2);
@@ -1019,19 +1078,104 @@ NSReturnAddress(int offset)
 	  _NS_RETURN_HACK(93); _NS_RETURN_HACK(94); _NS_RETURN_HACK(95);
 	  _NS_RETURN_HACK(96); _NS_RETURN_HACK(97); _NS_RETURN_HACK(98);
 	  _NS_RETURN_HACK(99);
-	  default: val = NULL; break;
+	  default: env->addr = NULL; break;
 	}
-      signal(SIGSEGV, old);
+      signal(SIGSEGV, env->segv);
+      signal(SIGBUS, env->bus);
     }
   else
     {
       env = jbuf();
-      memcpy(&old, env + 1, sizeof(old));
-      signal(SIGSEGV, old);
-      val = NULL;
+      signal(SIGSEGV, env->segv);
+      signal(SIGBUS, env->bus);
+      env->addr = NULL;
     }
 
-  return val;
+  return env->addr;
+}
+
+NSMutableArray *
+GSPrivateStackAddresses(void)
+{
+  unsigned              n = NSCountFrames();
+  NSMutableArray        *stack = [NSMutableArray arrayWithCapacity: n];
+  CREATE_AUTORELEASE_POOL(pool);
+  unsigned              i;
+  jbuf_type             *env;
+
+  /* There should be more frame addresses than return addresses.
+   */
+  if (n > 0)
+    {
+      n--;
+    }
+  if (n > 0)
+    {
+      n--;
+    }
+
+  env = jbuf();
+  if (setjmp(env->buf) == 0)
+    {
+      env->segv = signal(SIGSEGV, recover);
+      env->bus = signal(SIGBUS, recover);
+
+      for (i = 0; i < n; i++)
+        {
+          switch (i)
+            {
+              _NS_RETURN_HACK(0); _NS_RETURN_HACK(1); _NS_RETURN_HACK(2);
+              _NS_RETURN_HACK(3); _NS_RETURN_HACK(4); _NS_RETURN_HACK(5);
+              _NS_RETURN_HACK(6); _NS_RETURN_HACK(7); _NS_RETURN_HACK(8);
+              _NS_RETURN_HACK(9); _NS_RETURN_HACK(10); _NS_RETURN_HACK(11);
+              _NS_RETURN_HACK(12); _NS_RETURN_HACK(13); _NS_RETURN_HACK(14);
+              _NS_RETURN_HACK(15); _NS_RETURN_HACK(16); _NS_RETURN_HACK(17);
+              _NS_RETURN_HACK(18); _NS_RETURN_HACK(19); _NS_RETURN_HACK(20);
+              _NS_RETURN_HACK(21); _NS_RETURN_HACK(22); _NS_RETURN_HACK(23);
+              _NS_RETURN_HACK(24); _NS_RETURN_HACK(25); _NS_RETURN_HACK(26);
+              _NS_RETURN_HACK(27); _NS_RETURN_HACK(28); _NS_RETURN_HACK(29);
+              _NS_RETURN_HACK(30); _NS_RETURN_HACK(31); _NS_RETURN_HACK(32);
+              _NS_RETURN_HACK(33); _NS_RETURN_HACK(34); _NS_RETURN_HACK(35);
+              _NS_RETURN_HACK(36); _NS_RETURN_HACK(37); _NS_RETURN_HACK(38);
+              _NS_RETURN_HACK(39); _NS_RETURN_HACK(40); _NS_RETURN_HACK(41);
+              _NS_RETURN_HACK(42); _NS_RETURN_HACK(43); _NS_RETURN_HACK(44);
+              _NS_RETURN_HACK(45); _NS_RETURN_HACK(46); _NS_RETURN_HACK(47);
+              _NS_RETURN_HACK(48); _NS_RETURN_HACK(49); _NS_RETURN_HACK(50);
+              _NS_RETURN_HACK(51); _NS_RETURN_HACK(52); _NS_RETURN_HACK(53);
+              _NS_RETURN_HACK(54); _NS_RETURN_HACK(55); _NS_RETURN_HACK(56);
+              _NS_RETURN_HACK(57); _NS_RETURN_HACK(58); _NS_RETURN_HACK(59);
+              _NS_RETURN_HACK(60); _NS_RETURN_HACK(61); _NS_RETURN_HACK(62);
+              _NS_RETURN_HACK(63); _NS_RETURN_HACK(64); _NS_RETURN_HACK(65);
+              _NS_RETURN_HACK(66); _NS_RETURN_HACK(67); _NS_RETURN_HACK(68);
+              _NS_RETURN_HACK(69); _NS_RETURN_HACK(70); _NS_RETURN_HACK(71);
+              _NS_RETURN_HACK(72); _NS_RETURN_HACK(73); _NS_RETURN_HACK(74);
+              _NS_RETURN_HACK(75); _NS_RETURN_HACK(76); _NS_RETURN_HACK(77);
+              _NS_RETURN_HACK(78); _NS_RETURN_HACK(79); _NS_RETURN_HACK(80);
+              _NS_RETURN_HACK(81); _NS_RETURN_HACK(82); _NS_RETURN_HACK(83);
+              _NS_RETURN_HACK(84); _NS_RETURN_HACK(85); _NS_RETURN_HACK(86);
+              _NS_RETURN_HACK(87); _NS_RETURN_HACK(88); _NS_RETURN_HACK(89);
+              _NS_RETURN_HACK(90); _NS_RETURN_HACK(91); _NS_RETURN_HACK(92);
+              _NS_RETURN_HACK(93); _NS_RETURN_HACK(94); _NS_RETURN_HACK(95);
+              _NS_RETURN_HACK(96); _NS_RETURN_HACK(97); _NS_RETURN_HACK(98);
+              _NS_RETURN_HACK(99);
+              default: env->addr = 0; break;
+            }
+          if (env->addr == 0)
+            {
+              break;
+            }
+          [stack addObject: [NSValue valueWithPointer: env->addr]];
+        }
+      signal(SIGSEGV, env->segv);
+      signal(SIGBUS, env->bus);
+    }
+  else
+    {
+      signal(SIGSEGV, env->segv);
+      signal(SIGBUS, env->bus);
+    }
+  RELEASE(pool);
+  return stack;
 }
 
 
