@@ -10,7 +10,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,9 +23,10 @@
    Boston, MA 02111 USA.
 
    <title>NSDecimal class reference</title>
-   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
+   $Date: 2008-06-12 11:44:00 +0100 (Thu, 12 Jun 2008) $ $Revision: 26630 $
    */
 
+#define _GNU_SOURCE
 #include <math.h>
 #if !defined(__APPLE__) || !defined(GNU_RUNTIME)
 #include <ctype.h>
@@ -35,6 +36,10 @@
 #include "Foundation/NSDictionary.h"
 #include "Foundation/NSUserDefaults.h"
 
+#ifndef NAN
+#define NAN 0.0
+#endif 
+
 /*
   This file provides two implementations of the NSDecimal functions.
   One is based on pure simple decimal mathematics, as we all learned it
@@ -780,9 +785,10 @@ NSDecimalPower(NSDecimal *result, const NSDecimal *n, unsigned power, NSRounding
 NSCalculationError
 NSDecimalMultiplyByPowerOf10(NSDecimal *result, const NSDecimal *n, short power, NSRoundingMode mode)
 {
-  int p = result->exponent + power;
+  int p;
 
   NSDecimalCopy(result, n);
+  p = result->exponent + power;
   if (p > 127)
     {
       result->validNumber = NO;
@@ -910,8 +916,7 @@ GSDecimalDouble(GSDecimal *number)
   int i;
 
   if (!number->validNumber)
-    // Somehow I dont have NAN defined on my machine
-    return 0.0;
+    return NAN;
 
   // Sum up the digits
   for (i = 0; i < number->length; i++)
