@@ -10,7 +10,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,14 +23,13 @@
    Boston, MA 02111 USA.
 
    <title>NSDictionary class reference</title>
-   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
+   $Date: 2008-06-12 11:44:00 +0100 (Thu, 12 Jun 2008) $ $Revision: 26630 $
    */
 
 #include "config.h"
 #include "Foundation/NSDictionary.h"
 #include "Foundation/NSArray.h"
 #include "Foundation/NSData.h"
-#include "Foundation/NSUtilities.h"
 #include "Foundation/NSString.h"
 #include "Foundation/NSException.h"
 #include "Foundation/NSAutoreleasePool.h"
@@ -450,8 +449,9 @@ static SEL	appSel;
 
 /**
  * Initialises a dictionary created using the list given as argument.
- * The list is alternately composed of objects and keys.
- * Thus, the list's length must be pair.
+ * The list is alternately composed of objects and keys and
+ * terminated by nil.  Thus, the list's length must be even,
+ * followed by nil.
  */
 - (id) initWithObjectsAndKeys: (id)firstObject, ...
 {
@@ -462,8 +462,9 @@ static SEL	appSel;
 
 /**
  * Returns a dictionary created using the list given as argument.
- * The list is alternately composed of objects and keys.
- * Thus, the list's length must be pair.
+ * The list is alternately composed of objects and keys and
+ * terminated by nil.  Thus, the list's length must be even,
+ * followed by nil.
  */
 + (id) dictionaryWithObjectsAndKeys: (id)firstObject, ...
 {
