@@ -9,7 +9,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSDistributedNotificationCenter class reference</title>
-   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
+   $Date: 2008-06-12 11:44:00 +0100 (Thu, 12 Jun 2008) $ $Revision: 26630 $
    */
 
 #include	"config.h"
@@ -42,7 +42,7 @@
 #include	"Foundation/NSHost.h"
 #include	"Foundation/NSPortNameServer.h"
 #include	"Foundation/NSDebug.h"
-
+#include        "Foundation/NSThread.h"
 #include	"../Tools/gdnc.h"
 
 
@@ -739,9 +739,13 @@ static NSDistributedNotificationCenter	*netCenter = nil;
 	  limit = [NSDate dateWithTimeIntervalSinceNow: 5.0];
 	  while (_remote == nil && [limit timeIntervalSinceNow] > 0)
 	    {
+              CREATE_AUTORELEASE_POOL(pool);
+              [NSThread sleepForTimeInterval: 0.05];
 	      _remote = [NSConnection
 		rootProxyForConnectionWithRegisteredName: service
 		host: host usingNameServer: ns];
+              RETAIN(_remote);
+              DESTROY(pool);
 	    }
 	  if (_remote == nil)
 	    {
@@ -751,8 +755,11 @@ static NSDistributedNotificationCenter	*netCenter = nil;
 		@"I attempted to start it at '%@'\n", cmd];
 	    }
 	}
+      else
+        {
+          RETAIN(_remote);
+        }
 
-      RETAIN(_remote);
       c = [_remote connectionForProxy];
       [_remote setProtocolForProxy: p];
     
