@@ -9,7 +9,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,13 +22,13 @@
    Boston, MA 02111 USA.
 
    <title>NSEnumerator class reference</title>
-   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
+   $Date: 2008-06-12 11:44:00 +0100 (Thu, 12 Jun 2008) $ $Revision: 26630 $
 */
 
-#include "config.h"
-#include "GNUstepBase/preface.h"
-#include "Foundation/NSUtilities.h"
-#include "Foundation/NSArray.h"
+#import "config.h"
+#import "GNUstepBase/preface.h"
+#import "Foundation/NSArray.h"
+#import "Foundation/NSEnumerator.h"
 
 /**
  *  Simple class for iterating over a collection of objects, usually returned
@@ -37,7 +37,9 @@
 @implementation NSEnumerator
 
 /**
- *  Returns all objects remaining in the enumeration as an array.
+ *  Returns all objects remaining in the enumeration as an array.<br />
+ *  Calling this method 'exhausts' the enumerator, leaving it at the
+ *  end of the collection being enumerated.
  */
 - (NSArray*) allObjects
 {
