@@ -11,7 +11,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,19 +24,20 @@
    Boston, MA 02111 USA.
 
    <title>NSHost class reference</title>
-   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
+   $Date: 2008-06-12 11:44:00 +0100 (Thu, 12 Jun 2008) $ $Revision: 26630 $
   */
 
-#include "config.h"
-#include "GNUstepBase/preface.h"
-#include "Foundation/NSLock.h"
-#include "Foundation/NSHost.h"
-#include "Foundation/NSArray.h"
-#include "Foundation/NSDictionary.h"
-#include "Foundation/NSSet.h"
-#include "Foundation/NSString.h"
-#include "Foundation/NSCoder.h"
-#include "Foundation/NSDebug.h"
+#import "config.h"
+#import "GNUstepBase/preface.h"
+#import "Foundation/NSLock.h"
+#import "Foundation/NSHost.h"
+#import "Foundation/NSArray.h"
+#import "Foundation/NSDictionary.h"
+#import "Foundation/NSEnumerator.h"
+#import "Foundation/NSSet.h"
+#import "Foundation/NSString.h"
+#import "Foundation/NSCoder.h"
+#import "Foundation/NSDebug.h"
 
 #if defined(__MINGW32__)
 #include <winsock2.h>
@@ -372,6 +373,10 @@ myHostName()
 	    }
 	}
     }
+  else
+    {
+      AUTORELEASE(RETAIN(host));
+    }
   [_hostCacheLock unlock];
   if (tryByAddress == YES)
     {
@@ -400,7 +405,6 @@ myHostName()
     {
       host = [_hostCache objectForKey: address];
     }
-
   if (host == nil)
     {
       struct hostent	*h;
@@ -436,6 +440,10 @@ myHostName()
 	  AUTORELEASE(host);
 	}
     }
+  else
+    {
+      AUTORELEASE(RETAIN(host));
+    }
   [_hostCacheLock unlock];
   return host;
 }
