@@ -9,7 +9,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -741,6 +741,18 @@ static NSDictionary *makeReference(unsigned ref)
 	  NSStringFromClass([self class]), NSStringFromSelector(_cmd)];
 	return;
 
+      case _C_ARY_B:
+	{
+	  int		count = atoi(++type);
+
+	  while (isdigit(*type))
+	    {
+	      type++;
+	    }
+	  [self encodeArrayOfObjCType: type count: count at: address];
+	}
+	return;
+
       default:	/* Types that can be ignored in first pass.	*/
 	[NSException raise: NSInvalidArgumentException
 		    format: @"-[%@ %@]: unknown type encoding ('%c')",
@@ -770,6 +782,16 @@ static NSDictionary *makeReference(unsigned ref)
   [_delegate archiverDidFinish: self];
 }
 
+- (id) init
+{
+  Class c = [self class];
+  RELEASE(self);
+  [NSException raise: NSInvalidArgumentException
+              format: @"-[%@ init]: cannot use -init for initialisation",
+              NSStringFromClass(c)];
+  return nil;
+}
+
 - (id) initForWritingWithMutableData: (NSMutableData*)data
 {
   self = [super init];
