@@ -10,7 +10,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,7 +23,7 @@
    Boston, MA 02111 USA.
 
    <title>NSLock class reference</title>
-   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
+   $Date: 2008-06-12 11:44:00 +0100 (Thu, 12 Jun 2008) $ $Revision: 26630 $
 */
 
 #include "config.h"
@@ -167,6 +167,14 @@ NSString *NSRecursiveLockException = @"NSRecursiveLockException";
   [super dealloc];
 }
 
+- (NSString*) description
+{
+  if (_name == nil)
+    return [super description];
+  return [NSString stringWithFormat: @"%@ named '%@'",
+    [super description], _name];
+}
+
 - (void) gcFinalize
 {
   if (_mutex != 0)
@@ -175,10 +183,21 @@ NSString *NSRecursiveLockException = @"NSRecursiveLockException";
       // If there are outstanding locks then it will block
       if (objc_mutex_deallocate(_MUTEX) == -1)
 	{
-	  NSWarnMLog(@"objc_mutex_deallocate() failed");
+	  NSWarnMLog(@"objc_mutex_deallocate() failed for %@", self);
 	}
       _mutex = 0;
     }
+  DESTROY(_name);
+}
+
+- (NSString*) name
+{
+  return _name;
+}
+
+- (void) setName: (NSString*)name
+{
+  ASSIGNCOPY(_name, name);
 }
 
 /**
@@ -304,6 +323,14 @@ NSString *NSRecursiveLockException = @"NSRecursiveLockException";
   [super dealloc];
 }
 
+- (NSString*) description
+{
+  if (_name == nil)
+    return [super description];
+  return [NSString stringWithFormat: @"%@ named '%@'",
+    [super description], _name];
+}
+
 - (void) gcFinalize
 {
   if (_condition != 0)
@@ -311,7 +338,7 @@ NSString *NSRecursiveLockException = @"NSRecursiveLockException";
       // Ask the runtime to deallocate the condition
       if (objc_condition_deallocate(_CONDITION) == -1)
 	{
-	  NSWarnMLog(@"objc_condition_deallocate() failed");
+	  NSWarnMLog(@"objc_condition_deallocate() failed for %@", self);
 	}
     }
   if (_mutex != 0)
@@ -320,9 +347,10 @@ NSString *NSRecursiveLockException = @"NSRecursiveLockException";
       // If there are outstanding locks then it will block
       if (objc_mutex_deallocate(_MUTEX) == -1)
 	{
-	  NSWarnMLog(@"objc_mutex_deallocate() failed");
+	  NSWarnMLog(@"objc_mutex_deallocate() failed for %@", self);
 	}
     }
+  DESTROY(_name);
 }
 
 // Return the current condition of the lock
@@ -354,6 +382,16 @@ NSString *NSRecursiveLockException = @"NSRecursiveLockException";
     }
 }
 
+- (NSString*) name
+{
+  return _name;
+}
+
+- (void) setName: (NSString*)name
+{
+  ASSIGNCOPY(_name, name);
+}
+
 - (void) unlockWithCondition: (int)value
 {
   int depth;
@@ -600,6 +638,14 @@ NSString *NSRecursiveLockException = @"NSRecursiveLockException";
   [super dealloc];
 }
 
+- (NSString*) description
+{
+  if (_name == nil)
+    return [super description];
+  return [NSString stringWithFormat: @"%@ named '%@'",
+    [super description], _name];
+}
+
 - (void) gcFinalize
 {
   if (_mutex != 0)
@@ -608,10 +654,21 @@ NSString *NSRecursiveLockException = @"NSRecursiveLockException";
       // If there are outstanding locks then it will block
       if (objc_mutex_deallocate(_MUTEX) == -1)
 	{
-	  NSWarnMLog(@"objc_mutex_deallocate() failed");
+	  NSWarnMLog(@"objc_mutex_deallocate() failed for %@", self);
 	}
       _mutex = 0;
     }
+  DESTROY(_name);
+}
+
+- (NSString*) name
+{
+  return _name;
+}
+
+- (void) setName: (NSString*)name
+{
+  ASSIGNCOPY(_name, name);
 }
 
 /**
