@@ -9,7 +9,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -51,7 +51,7 @@
 #include <stdlib.h>
 
 #ifdef HAVE_UNISTD_H
-#include <unistd.h>		/* for gethostname() */
+#include <unistd.h>
 #endif
 
 #include <sys/param.h>		/* for MAXHOSTNAMELEN */
@@ -1843,6 +1843,7 @@ typedef	struct {
        * a maximum of NETBLOCK bytes.  This is to try to get a single,
        * efficient write operation if possible.
        */
+      c = [components count];
       for (i = 1; i < c; i++)
 	{
 	  id	o = [components objectAtIndex: i];
