@@ -11,7 +11,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,7 +24,7 @@
    Boston, MA 02111 USA.
 
    <title>NSMethodSignature class reference</title>
-   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
+   $Date: 2008-06-12 11:44:00 +0100 (Thu, 12 Jun 2008) $ $Revision: 26630 $
    */
 
 #include "config.h"
@@ -170,13 +170,24 @@
   if (_info == 0)
     {
       const char	*types = _methodTypes;
+      char		*outTypes;
       unsigned int	i;
 
-      _info = NSZoneMalloc(NSDefaultMallocZone(),
-	sizeof(NSArgumentInfo)*(_numArgs+1));
+      /* Allocate space enough for an NSArgumentInfo structure for each
+       * argument (including the return type), and enough space to hold
+       * the type information for each argument as a nul terminated
+       * string.
+       */
+      outTypes = NSZoneMalloc(NSDefaultMallocZone(),
+	sizeof(NSArgumentInfo)*(_numArgs+1) + strlen(types)*2);
+      _info = (NSArgumentInfo*)outTypes;
+      outTypes = outTypes + sizeof(NSArgumentInfo)*(_numArgs+1);
+      /* Fill in the full argment information for each arg.
+       */
       for (i = 0; i <= _numArgs; i++)
 	{
-	  types = mframe_next_arg(types, &_info[i]);
+	  types = mframe_next_arg(types, &_info[i], outTypes);
+	  outTypes += strlen(outTypes) + 1;
 	}
     }
   return _info;
