@@ -9,7 +9,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
    
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,6 +23,8 @@
    */ 
 
 #import "Foundation/NSNetServices.h"
+#import "Foundation/NSDictionary.h"
+#import "Foundation/NSEnumerator.h"
 #import "Foundation/NSData.h"
 #import "Foundation/NSDebug.h"
 #import "Foundation/NSNull.h"
@@ -2332,6 +2334,15 @@ static void DNSSD_API
   return [((Service*)_reserved)->info objectForKey: @"Name"];
 }
 
+- (NSInteger) port
+{
+  return 0;
+}
+
+- (void) publishWithOptions: (NSNetServiceOptions)options
+{
+}
+
 /**
  * Returns the type of the service.
  *
