@@ -11,7 +11,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,7 +24,7 @@
    Boston, MA 02111 USA.
 
    <title>NSNumberFormatter class reference</title>
-   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
+   $Date: 2008-06-12 11:44:00 +0100 (Thu, 12 Jun 2008) $ $Revision: 26630 $
    */
 
 #include "Foundation/NSAttributedString.h"
@@ -36,6 +36,8 @@
 #include "Foundation/NSUserDefaults.h"
 #include "Foundation/NSCharacterSet.h"
 
+#include "GNUstepBase/GSLocale.h"
+
 @implementation NSNumberFormatter
 
 - (BOOL) allowsFloats
@@ -534,7 +536,26 @@
   BOOL			displayFractionalPart = NO;
   BOOL			negativeNumber = NO;
   NSString		*useFormat;
+  NSString		*defaultDecimalSeparator = nil;
+  NSString		*defaultThousandsSeparator = nil;
 
+  if (_localizesFormat)
+    {
+      NSDictionary *defaultLocale = GSDomainFromDefaultLocale();
+      defaultDecimalSeparator 
+	= [defaultLocale objectForKey: NSDecimalSeparator];
+      defaultThousandsSeparator 
+	= [defaultLocale objectForKey: NSThousandsSeparator];
+    }
+
+  if (defaultDecimalSeparator == nil)
+    {
+      defaultDecimalSeparator = @".";
+    }
+  if (defaultThousandsSeparator == nil)
+    {
+      defaultThousandsSeparator = @",";
+    }
   formattingCharacters = [NSCharacterSet
     characterSetWithCharactersInString: @"0123456789#.,_"];
   placeHolders = [NSCharacterSet 
@@ -546,7 +567,8 @@
     return [[self attributedStringForNotANumber] string];
   if ([anObject isEqual: [NSDecimalNumber notANumber]])
     return [[self attributedStringForNotANumber] string];
-  if ([anObject isEqual: [NSDecimalNumber zero]])
+  if (_attributedStringForZero
+      && [anObject isEqual: [NSDecimalNumber zero]])
     return [[self attributedStringForZero] string];
   
   useFormat = _positiveFormat;
@@ -560,7 +582,10 @@
   // if no format specified, use the same default that Cocoa does
   if (nil == useFormat)
     {
-      useFormat = negativeNumber ? @"-#,###.##" : @"#,###.##";
+      useFormat = [NSString stringWithFormat: @"%@#%@###%@##",
+			    negativeNumber ? @"-" : @"",
+			    defaultThousandsSeparator,
+			    defaultDecimalSeparator];
     }
 
   prefixRange = [useFormat rangeOfCharacterFromSet: formattingCharacters];
@@ -580,15 +605,16 @@
   //should also set NSDecimalDigits?
   
   if ([self hasThousandSeparators]
-    && (0 != [useFormat rangeOfString:@","].length))
+    && (0 != [useFormat rangeOfString: defaultThousandsSeparator].length))
     {
       displayThousandsSeparators = YES;
     }
 
   if ([self allowsFloats]
-    && (NSNotFound != [useFormat rangeOfString:@"." ].location))
+    && (NSNotFound 
+	!= [useFormat rangeOfString: defaultDecimalSeparator].location))
     {
-      decimalPlaceRange = [useFormat rangeOfString: @"."
+      decimalPlaceRange = [useFormat rangeOfString: defaultDecimalSeparator
 					   options: NSBackwardsSearch];
       if (NSMaxRange(decimalPlaceRange) == [useFormat length])
         {
@@ -636,14 +662,15 @@
       while (([placeHolders characterIsMember:
         [useFormat characterAtIndex: NSMaxRange(intPartRange)]]
         || [[useFormat substringFromRange:
-          NSMakeRange(NSMaxRange(intPartRange), 1)] isEqual: @","])
+          NSMakeRange(NSMaxRange(intPartRange), 1)] isEqual:
+	    defaultThousandsSeparator])
         && NSMaxRange(intPartRange) < [useFormat length] - 1)
         {
           intPartRange.length++;
         }
     }
   intPad = [[useFormat substringWithRange: intPartRange] mutableCopy];
-  [intPad replaceOccurrencesOfString: @","
+  [intPad replaceOccurrencesOfString: defaultThousandsSeparator
     withString: @""
     options: 0
     range: NSMakeRange(0, [intPad length])];
