@@ -9,7 +9,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSObjCRuntime class reference</title>
-   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
+   $Date: 2008-06-12 11:44:00 +0100 (Thu, 12 Jun 2008) $ $Revision: 26630 $
    */
 
 #include "config.h"
@@ -34,7 +34,39 @@
 #include <string.h>
 
 /**
- * Returns a string object containing the method name for
+ * Returns a string object containing the name for
+ * aProtocol.  If aProtocol is 0, returns nil.
+ */
+NSString *
+NSStringFromProtocol(Protocol *aProtocol)
+{
+  if (aProtocol != (Protocol*)0)
+    return [NSString stringWithUTF8String: (const char*)[aProtocol name]];
+  return nil;
+}
+
+/**
+ * Returns the protocol whose name is supplied in the
+ * aProtocolName argument, or 0 if a nil string is supplied.
+ */
+Protocol *   
+NSProtocolFromString(NSString *aProtocolName)
+{
+  if (aProtocolName != nil)
+    {
+      int	len = [aProtocolName length];
+      char	buf[len+1];
+
+      [aProtocolName getCString: buf
+		      maxLength: len + 1
+		       encoding: NSASCIIStringEncoding];
+      return GSProtocolFromName (buf);
+    }
+  return (Protocol*)0;
+}
+
+/**
+ * Returns a string object containing the name for
  * aSelector.  If aSelector is 0, returns nil.
  */
 NSString *
@@ -46,7 +78,7 @@ NSStringFromSelector(SEL aSelector)
 }
 
 /**
- * Returns a selector for the method whose name is supplied in the
+ * Returns the selector whose name is supplied in the
  * aSelectorName argument, or 0 if a nil string is supplied.
  */
 SEL
@@ -109,7 +141,7 @@ const char *
 NSGetSizeAndAlignment(const char *typePtr, unsigned *sizep, unsigned *alignp)
 {
   NSArgumentInfo	info;
-  typePtr = mframe_next_arg(typePtr, &info);
+  typePtr = mframe_next_arg(typePtr, &info, 0);
   if (sizep)
     *sizep = info.size;
   if (alignp)
