@@ -9,7 +9,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSObject class reference</title>
-   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
+   $Date: 2008-06-12 11:44:00 +0100 (Thu, 12 Jun 2008) $ $Revision: 26630 $
    */
 
 /* On some versions of mingw we need to work around bad function declarations
@@ -230,7 +230,7 @@ typedef int32_t volatile *gsatomic_t;
 #endif
 
 
-#if	defined(__linux__) && (defined(__i386__) || defined(__x86_64__))
+#elif	defined(__linux__) && (defined(__i386__) || defined(__x86_64__))
 /* Set up atomic read, increment and decrement for intel style linux
  */
 
@@ -241,26 +241,59 @@ typedef int32_t volatile *gsatomic_t;
 static __inline__ int
 GSAtomicIncrement(gsatomic_t X)
 {
-  int	i = 1;
-  __asm__ __volatile__(
-  "lock ; xaddl %0, %1;"
-  :"=r"(i)
-  :"m"(*X), "0"(i));
-  return i + 1;
+ __asm__ __volatile__ (
+     "lock addl $1, %0"
+     :"=m" (*X));
+ return *X;
 }
 
 static __inline__ int
 GSAtomicDecrement(gsatomic_t X)
 {
-  int	i = -1;
-  __asm__ __volatile__(
-  "lock ; xaddl %0, %1;"
-  :"=r"(i)
-  :"m"(*X), "0"(i));
-  return i - 1;
+ __asm__ __volatile__ (
+     "lock subl $1, %0"
+     :"=m" (*X));
+ return *X;
+}
+
+#elif defined(__PPC__)
+
+typedef int32_t volatile *gsatomic_t;
+
+#define	GSATOMICREAD(X)	(*(X))
+
+static __inline__ int
+GSAtomicIncrement(gsatomic_t X)
+{
+  int tmp;
+  __asm__ __volatile__ (
+    "modified:"
+    "lwarx %0,0,%1 \n"
+    "addic %0,%0,1 \n"
+    "stwcx. %0,0,%1 \n"
+    "bne- modified \n"
+    :"=&r" (tmp)
+    :"r" (X)
+    :"cc", "memory");
+  return *X;
+}
+
+static __inline__ int
+GSAtomicDecrement(gsatomic_t X)
+{
+  int tmp;
+  __asm__ __volatile__ (
+    "modified:"
+    "lwarx %0,0,%1 \n"
+    "addic %0,%0,-1 \n"
+    "stwcx. %0,0,%1 \n"
+    "bne- modified \n"
+    :"=&r" (tmp)
+    :"r" (X)
+    :"cc", "memory");
+  return *X;
 }
 
-#endif
 #endif
 
 #if	defined(REFCNT_LOCAL) && !defined(GSATOMICREAD)
