@@ -9,7 +9,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSPort class reference</title>
-   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
+   $Date: 2008-06-12 11:44:00 +0100 (Thu, 12 Jun 2008) $ $Revision: 26630 $
    */
 
 #include "config.h"
@@ -49,11 +49,16 @@
 
 @implementation NSPort
 
-/**
- *  Exception raised if a timeout occurs during a port send or receive
- *  operation.
- */
-NSString * const NSPortTimeoutException = @"NSPortTimeoutException";
+NSString * const NSInvalidReceivePortException
+  = @"NSInvalidReceivePortException";
+NSString * const NSInvalidSendPortException
+  = @"NSInvalidSendPortException";
+NSString * const NSPortReceiveException
+  = @"NSPortReceiveException";
+NSString * const NSPortSendException
+  = @"NSPortSendException";
+NSString * const NSPortTimeoutException
+  = @"NSPortTimeoutException";
 
 static Class	NSPort_abstract_class;
 static Class	NSPort_concrete_class;
