@@ -9,7 +9,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,18 +22,19 @@
    Boston, MA 02111 USA.
 
    <title>NSPortNameServer class reference</title>
-   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
+   $Date: 2008-06-12 11:44:00 +0100 (Thu, 12 Jun 2008) $ $Revision: 26630 $
    */
 
-#include "config.h"
-#include "Foundation/NSString.h"
-#include "Foundation/NSException.h"
-#include "Foundation/NSPort.h"
-#include "Foundation/NSPortNameServer.h"
-#include "Foundation/NSDebug.h"
-#include "Foundation/NSLock.h"
-#include "Foundation/NSUserDefaults.h"
-#include "GSPortPrivate.h"
+#import "config.h"
+#import "Foundation/NSString.h"
+#import "Foundation/NSException.h"
+#import "Foundation/NSEnumerator.h"
+#import "Foundation/NSPort.h"
+#import "Foundation/NSPortNameServer.h"
+#import "Foundation/NSDebug.h"
+#import "Foundation/NSLock.h"
+#import "Foundation/NSUserDefaults.h"
+#import "GSPortPrivate.h"
 
 
 /**
