@@ -10,7 +10,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,7 +23,7 @@
    Boston, MA 02111 USA.
 
    <title>NSProcessInfo class reference</title>
-   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
+   $Date: 2008-06-12 11:44:00 +0100 (Thu, 12 Jun 2008) $ $Revision: 26630 $
 */
 
 /*************************************************************************
@@ -67,6 +67,18 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+
+#ifdef HAVE_WINDOWS_H
+#  include <windows.h>
+#endif
+
+#ifdef  HAVE_SYS_SIGNAL_H
+#include <sys/signal.h>
+#endif
+#ifdef  HAVE_SIGNAL_H
+#include <signal.h>
+#endif
+
 #include <sys/file.h>
 #ifdef HAVE_SYS_FCNTL_H
 #include <sys/fcntl.h>
@@ -768,6 +780,10 @@ static char	**_gnu_noobjc_env = NULL;
 
  proc_fs_error:
 #ifdef HAVE_STRERROR
+  /* Don't care about thread safety of strerror() here as this is only
+   * called in the initial thread and there shouldn't be any other
+   * threads at this point.
+   */
   fprintf(stderr, "Couldn't open file %s when starting gnustep-base; %s\n",
 	   proc_file_name, strerror(errno));
 #else  /* !HAVE_FUNCTION_STRERROR */
@@ -919,6 +935,28 @@ int main(int argc, char *argv[], char *env[])
   return _gnu_sharedProcessInfoObject;
 }
 
++ (BOOL) _exists: (int)pid
+{
+  if (pid > 0)
+    {
+#if	defined(__MINGW32__)
+      HANDLE        h = OpenProcess(PROCESS_QUERY_INFORMATION,0,pid);
+      if (h == NULL && GetLastError() != ERROR_ACCESS_DENIED)
+        {
+          return NO;
+        }
+      CloseHandle(h);
+#else
+      if (kill(pid, 0) < 0 && errno == ESRCH)
+        {
+          return NO;
+        }
+#endif
+      return YES;
+    }
+  return NO;
+}
+
 - (NSArray *) arguments
 {
   return _gnu_arguments;
@@ -1154,6 +1192,20 @@ static void determineOperatingSystem()
   return;
 }
 
+- (NSUInteger) processorCount
+{
+  return 0;     // FIXME
+}
+
+- (NSUInteger) activeProcessorCount
+{
+  return 0;     // FIXME
+}
+
+- (unsigned long long) physicalMemory
+{
+  return 0;     // FIXME
+}
 @end
 
 @implementation	NSProcessInfo (GNUstep)
