@@ -11,7 +11,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,7 +24,7 @@
    Boston, MA 02111 USA.
 
    <title>NSScanner class reference</title>
-   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
+   $Date: 2008-06-12 11:44:00 +0100 (Thu, 12 Jun 2008) $ $Revision: 26630 $
 */
 
 /* We need to define _GNU_SOURCE on systems (SuSE) to get LONG_LONG_MAX.  */
@@ -1147,6 +1147,22 @@ typedef struct {
   return n;
 }
 
+- (BOOL) scanHexDouble: (double *)result
+{
+  return NO;    // FIXME
+}
+- (BOOL) scanHexFloat: (float *)result
+{
+  return NO;    // FIXME
+}
+- (BOOL) scanHexLongLong: (unsigned long long *)result
+{
+  return NO;    // FIXME
+}
+- (BOOL) scanInteger: (NSInteger *)value
+{
+  return NO;    // FIXME
+}
 @end
 
 /*
