@@ -11,8 +11,8 @@
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
-   as published by the Free Software Foundation; either version 3 of
-   the License, or (at your option) any later version.
+   as published by the Free Software Foundation; either
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,7 +25,7 @@
    Boston, MA 02111 USA.
 
    <title>NSTimeZone class reference</title>
-   $Date: 2007-11-06 03:27:56 -0700 (Tue, 06 Nov 2007) $ $Revision: 25562 $
+   $Date: 2008-06-12 11:44:00 +0100 (Thu, 12 Jun 2008) $ $Revision: 26630 $
  */
 
 /* Use the system time zones if available. In other cases, use an
@@ -96,7 +96,6 @@
 #include "Foundation/NSProcessInfo.h"
 #include "Foundation/NSString.h"
 #include "Foundation/NSUserDefaults.h"
-#include "Foundation/NSUtilities.h"
 #include "Foundation/NSZone.h"
 #include "Foundation/NSBundle.h"
 #include "Foundation/NSMapTable.h"
@@ -117,6 +116,9 @@
 #include "nstzfile.h"
 #endif
 
+NSString * const NSSystemTimeZoneDidChangeNotification
+  = @"NSSystemTimeZoneDidChangeNotification";
+
 /* Key for local time zone in user defaults. */
 #define LOCALDBKEY @"Local Time Zone"
 
@@ -320,7 +322,8 @@ static NSString *_time_zone_path(NSString *subpath, NSString *type)
 
 /* Private methods for obtaining resource file names. */
 @interface NSTimeZone (Private)
-+ (NSString*) getTimeZoneFile: (NSString*)name;
++ (NSString*) _getTimeZoneFile: (NSString*)name;
++ (void) _notified: (NSNotification*)n;
 @end
 
 
@@ -466,7 +469,7 @@ static NSString *_time_zone_path(NSString *subpath, NSString *type)
 	    {
 	      NSString		*fileName;
 
-	      fileName = [NSTimeZoneClass getTimeZoneFile: name];
+	      fileName = [NSTimeZoneClass _getTimeZoneFile: name];
 	      if (fileName == nil
 		|| ![[NSFileManager defaultManager] fileExistsAtPath: fileName])
 #if	defined(__MINGW32__)
@@ -1287,6 +1290,11 @@ static NSMapTable	*absolutes = 0;
       localTimeZone = [[NSLocalTimeZone alloc] init];
 
       zone_mutex = [GSLazyRecursiveLock new];
+
+      [[NSNotificationCenter defaultCenter] addObserver: self
+        selector: @selector(_notified:)
+        name: NSUserDefaultsDidChangeNotification
+        object: nil];
     }
 }
 
@@ -1313,6 +1321,9 @@ static NSMapTable	*absolutes = 0;
     {
       [zone_mutex unlock];
     }
+  [[NSNotificationCenter defaultCenter]
+    postNotificationName: NSSystemTimeZoneDidChangeNotification
+                  object: nil];
 }
 
 /**
@@ -1579,7 +1590,7 @@ static NSMapTable	*absolutes = 0;
 	}
       else
 	{
-	  NSString	*zonedir = [NSTimeZone getTimeZoneFile: @"WET"]; 
+	  NSString	*zonedir = [NSTimeZone _getTimeZoneFile: @"WET"]; 
 
 	  if (tzdir != nil)
 	    {
@@ -1593,12 +1604,11 @@ static NSMapTable	*absolutes = 0;
 		{
 		  NSTimeZone	*zone = nil;
 		  BOOL		isDir;
-
-		  // FIXME: check file validity.
 		
 		  path = [zonedir stringByAppendingPathComponent: name];
 		  if ([mgr fileExistsAtPath: path isDirectory: &isDir]
-		    && isDir == NO)
+		      && isDir == NO
+		      && [[path pathExtension] isEqual: @"tab"] == NO)
 		    {
 		      zone = [zoneDictionary objectForKey: name];
 		      if (zone == nil)
@@ -1606,6 +1616,10 @@ static NSMapTable	*absolutes = 0;
 			  NSData	*data;
 
 			  data = [NSData dataWithContentsOfFile: path];
+			  /* We should really make sure this is a real
+			     zone file and not something extra that happens
+			     to be in this directory, but initWithName:data:
+			     will do this anyway and log a message if not. */
 			  zone = [[self alloc] initWithName: name data: data];
 			  AUTORELEASE(zone);
 			}
@@ -1932,6 +1946,32 @@ static NSMapTable	*absolutes = 0;
   return [self name];
 }
 
+- (NSTimeInterval) daylightSavingTimeOffsetForDate: (NSDate *)aDate
+{
+  return 0.0;   // FIXME
+}
+
+- (NSDate *) nextDaylightSavingTimeTransitionAfterDate: (NSDate *)aDate
+{
+  return nil;   // FIXME;
+}
+
+- (NSTimeInterval) daylightSavingTimeOffset
+{
+  return 0.0;   // FIXME
+}
+
+- (NSDate *) nextDaylightSavingTimeTransition
+{
+  return nil;   // FIXME;
+}
+
+- (NSString *)localizedName: (NSTimeZoneNameStyle)style
+                     locale: (NSLocale *)locale
+{
+  return nil;   // FIXME;
+}
+                    
 @end
 
 /**
@@ -1999,7 +2039,7 @@ static NSString *zoneDirs[] = {
 /**
  * Returns the path to the named zone info file.
  */
-+ (NSString*) getTimeZoneFile: (NSString *)name
++ (NSString*) _getTimeZoneFile: (NSString *)name
 {
   static BOOL	beenHere = NO;
   NSString	*dir = nil;
@@ -2049,6 +2089,20 @@ static NSString *zoneDirs[] = {
   return [dir stringByAppendingPathComponent: name];
 }
 
++ (void) _notified: (NSNotification*)n
+{
+  NSString      *name;
+
+  /* If the name of the system time zone has changed ...
+   * get a new system time zone.
+   */
+  name = [[NSUserDefaults standardUserDefaults] stringForKey: LOCALDBKEY];
+  if ([name length] > 0 && [name isEqual: [[self systemTimeZone] name]] == NO)
+    {
+      [self resetSystemTimeZone];
+      [self systemTimeZone];
+    }
+}
 @end
 
 
@@ -2113,173 +2167,192 @@ GSBreakTime(NSTimeInterval when, int *year, int *month, int *day,
 - (id) initWithName: (NSString*)name data: (NSData*)data
 {
   HKEY     regDirKey;
-  BOOL     isNT = NO,regFound=NO;
+  BOOL     isNT = NO, regFound=NO;
 
-  /* Open the key in the local machine hive where the time zone data is stored. */
+  /* Open the key in the local machine hive where
+   * the time zone data is stored. */
   if (ERROR_SUCCESS == RegOpenKeyExW(HKEY_LOCAL_MACHINE,
-  	L"SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Time Zones",
-  	0,
-  	KEY_READ,
-  	&regDirKey))
-  {
-    isNT=YES;
-    regFound=YES;
-  }
-  else
-  {
-    if (ERROR_SUCCESS == RegOpenKeyExW(HKEY_LOCAL_MACHINE,
-    	L"SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Time Zones",
-    	0,
-    	KEY_READ,
-    	&regDirKey))
+    L"SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Time Zones",
+    0,
+    KEY_READ,
+    &regDirKey))
     {
-    	regFound=YES;
+      isNT=YES;
+      regFound=YES;
+    }
+  else
+    {
+      if (ERROR_SUCCESS == RegOpenKeyExW(HKEY_LOCAL_MACHINE,
+          L"SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Time Zones",
+          0,
+          KEY_READ,
+          &regDirKey))
+        {
+          regFound=YES;
+        }
     }
-  }
 
   if (regFound)
-  {
-    	/* Iterate over all subKeys in the registry to find the right one.
-    	   Unfortunately name is a localized value. The keys in the registry are
-    	   unlocalized names. */
-	   wchar_t  achKey[255];              // buffer for subkey name
-    	DWORD    cbName;                   // size of name string 
-    	wchar_t  achClass[MAX_PATH] = L""; // buffer for class name 
-    	DWORD    cchClassName = MAX_PATH;  // size of class string 
-    	DWORD    cSubKeys=0;               // number of subkeys 
-    	DWORD    cbMaxSubKey;              // longest subkey size 
-    	DWORD    cchMaxClass;              // longest class string 
-    	DWORD    cValues;                  // number of values for key 
-    	DWORD    cchMaxValue;              // longest value name 
-    	DWORD    cbMaxValueData;           // longest value data 
-    	DWORD    cbSecurityDescriptor;     // size of security descriptor 
-    	FILETIME ftLastWriteTime;          // last write time 
+    {
+      /* Iterate over all subKeys in the registry to find the right one.
+         Unfortunately name is a localized value. The keys in the registry are
+         unlocalized names. */
+      wchar_t  achKey[255];              // buffer for subkey name
+      DWORD    cbName;                   // size of name string 
+      wchar_t  achClass[MAX_PATH] = L""; // buffer for class name 
+      DWORD    cchClassName = MAX_PATH;  // size of class string 
+      DWORD    cSubKeys=0;               // number of subkeys 
+      DWORD    cbMaxSubKey;              // longest subkey size 
+      DWORD    cchMaxClass;              // longest class string 
+      DWORD    cValues;                  // number of values for key 
+      DWORD    cchMaxValue;              // longest value name 
+      DWORD    cbMaxValueData;           // longest value data 
+      DWORD    cbSecurityDescriptor;     // size of security descriptor 
+      FILETIME ftLastWriteTime;          // last write time 
+      DWORD     i, retCode;
+      BOOL	tzFound = NO;
 		
-		DWORD i, retCode;
-		BOOL	tzFound = NO;
-		
-		/* Get the class name and the value count. */
-	    retCode = RegQueryInfoKeyW(
-	        regDirKey,               // key handle 
-	        achClass,                // buffer for class name 
-	        &cchClassName,           // size of class string 
-	        NULL,                    // reserved 
-	        &cSubKeys,               // number of subkeys 
-	        &cbMaxSubKey,            // longest subkey size 
-	        &cchMaxClass,            // longest class string 
-	        &cValues,                // number of values for this key 
-	        &cchMaxValue,            // longest value name 
-	        &cbMaxValueData,         // longest value data 
-	        &cbSecurityDescriptor,   // security descriptor 
-	        &ftLastWriteTime);       // last write time 
+      /* Get the class name and the value count. */
+      retCode = RegQueryInfoKeyW(
+        regDirKey,               // key handle 
+        achClass,                // buffer for class name 
+        &cchClassName,           // size of class string 
+        NULL,                    // reserved 
+        &cSubKeys,               // number of subkeys 
+        &cbMaxSubKey,            // longest subkey size 
+        &cchMaxClass,            // longest class string 
+        &cValues,                // number of values for this key 
+        &cchMaxValue,            // longest value name 
+        &cbMaxValueData,         // longest value data 
+        &cbSecurityDescriptor,   // security descriptor 
+        &ftLastWriteTime);       // last write time 
 
-    	if (cSubKeys && (retCode == ERROR_SUCCESS))
+      if (cSubKeys && (retCode == ERROR_SUCCESS))
     	{
-    		wchar_t *wName = malloc(([name length]+1) * sizeof(wchar_t));
-    		if (wName)
-    		{
-    			[name getCharacters:wName];
+          unsigned wLen = [name length];
+          wchar_t *wName = malloc((wLen+1) * sizeof(wchar_t));
 
-				for (i=0; i<cSubKeys && !tzFound; i++) 
-				{ 
-				   cbName = 255;
-				   
-				   retCode = RegEnumKeyExW(regDirKey, i, achKey, &cbName, NULL, NULL, NULL, &ftLastWriteTime);         
-				   if (retCode == ERROR_SUCCESS) 
-				   {
-				   	 wchar_t keyBuffer[16384];
-				   	 HKEY regKey;
-				       
-				       if (isNT)
-				       	wcscpy(keyBuffer,L"SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Time Zones\\");
-						 else
-						 	wcscpy(keyBuffer,L"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Time Zones\\");
-				       
-				       wcscat(keyBuffer,achKey);
-	 		          if (ERROR_SUCCESS == RegOpenKeyExW(HKEY_LOCAL_MACHINE, keyBuffer, 0, KEY_READ, &regKey))
-				       {
-					      wchar_t buf[256];
-					      wchar_t standardName[256];
-					      wchar_t daylightName[256];
-					      DWORD bufsize;
-					      DWORD type;
-	
-							/* check standardname */
-							standardName[0]=L'\0';
-					      bufsize=sizeof(buf);
-					      if (ERROR_SUCCESS==RegQueryValueExW(regKey, L"Std", 0, &type, (BYTE *)buf, &bufsize))
-					      {
-					      	wcscpy(standardName,buf);
-					      	if (wcscmp(standardName,wName) == 0)
-					      		tzFound = YES;
-					   	}
-			
-							/* check daylightname */
-							daylightName[0]=L'\0';
-					      bufsize=sizeof(buf);
-					      if (ERROR_SUCCESS==RegQueryValueExW(regKey, L"Dlt", 0, &type, (BYTE *)buf, &bufsize))
-					      {
-					      	wcscpy(daylightName,buf);
-					      	if (wcscmp(daylightName,wName) == 0)
-					      		tzFound = YES;
-					   	}
-	
-							if (tzFound)
-							{
-						      /* Read in the time zone data */
-						      bufsize=sizeof(buf);
-						      if (ERROR_SUCCESS==RegQueryValueExW(regKey, L"TZI", 0, &type, (BYTE *)buf, &bufsize))
-						      {
-						          TZI *tzi = (void*)buf;
-						          Bias = tzi->Bias;
-						          StandardBias = tzi->StandardBias;
-						          DaylightBias = tzi->DaylightBias;
-						          StandardDate = tzi->StandardDate;
-						          DaylightDate = tzi->DaylightDate;
-						      }
-						      
-						      /* Set the standard name for the time zone. */
-						      if (wcslen(standardName))
-						      {
-						          int a, b;
-						          ASSIGN(timeZoneName,[NSString stringWithCharacters:standardName length:wcslen(standardName)]);
-	
-						          /* Abbr generated here is IMHO a bit suspicous but I kept it */
-						          for(a=0,b=0;standardName[a];a++)
-						          {
-						            if (iswupper(standardName[a]))
-						              standardName[b++]=standardName[a];
-						          }
-						          standardName[b]=L'\0';
-						          ASSIGN(timeZoneNameAbbr,[NSString stringWithCharacters:standardName length:wcslen(standardName)]);
-						      }
-	
-						      /* Set the daylight savings name for the time zone. */
-						      if (wcslen(daylightName))
-						      {
-						          int a,b;
-						          ASSIGN(daylightZoneName,[NSString stringWithCharacters:daylightName length:wcslen(daylightName)]);
-	
-						          /* Abbr generated here is IMHO a bit suspicous but I kept it */
-						          for(a=0,b=0;daylightName[a];a++)
-						          {
-						              if (iswupper(daylightName[a]))
-						                daylightName[b++]=daylightName[a];
-						          }
-						          daylightName[b]=L'\0';
-						          ASSIGN(daylightZoneNameAbbr,[NSString stringWithCharacters:daylightName length:wcslen(daylightName)]);
-					      	}
-					   	}
-					      RegCloseKey(regKey);
-				   	}			       
-				   }
-				}
-				
-				free(wName);
-			}
-    	}
+          if (wName)
+            {
+              [name getCharacters: wName];
+              wName[wLen] = 0;
+              for (i = 0; i < cSubKeys && !tzFound; i++) 
+                { 
+                  cbName = 255;
+                   
+                  retCode = RegEnumKeyExW(regDirKey, i, achKey, &cbName,
+                    NULL, NULL, NULL, &ftLastWriteTime);         
+                  if (retCode == ERROR_SUCCESS) 
+                    {
+                      wchar_t keyBuffer[16384];
+                      HKEY regKey;
+                       
+                      if (isNT)
+                        wcscpy(keyBuffer, L"SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Time Zones\\");
+                      else
+                        wcscpy(keyBuffer, L"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Time Zones\\");
+                       
+                      wcscat(keyBuffer, achKey);
+                      if (ERROR_SUCCESS == RegOpenKeyExW(HKEY_LOCAL_MACHINE,
+                        keyBuffer, 0, KEY_READ, &regKey))
+                        {
+                          wchar_t buf[256];
+                          wchar_t standardName[256];
+                          wchar_t daylightName[256];
+                          DWORD bufsize;
+                          DWORD type;
+
+                          /* check standardname */
+                          standardName[0] = L'\0';
+                          bufsize = sizeof(buf);
+                          if (ERROR_SUCCESS == RegQueryValueExW(regKey,
+                            L"Std", 0, &type, (BYTE *)buf, &bufsize))
+                            {
+                              wcscpy(standardName, buf);
+                              if (wcscmp(standardName, wName) == 0)
+                                tzFound = YES;
+                            }
+        
+                          /* check daylightname */
+                          daylightName[0] = L'\0';
+                          bufsize = sizeof(buf);
+                          if (ERROR_SUCCESS == RegQueryValueExW(regKey,
+                            L"Dlt", 0, &type, (BYTE *)buf, &bufsize))
+                            {
+                              wcscpy(daylightName, buf);
+                              if (wcscmp(daylightName, wName) == 0)
+                                tzFound = YES;
+                            }
+
+                          if (tzFound)
+                            {
+                              /* Read in the time zone data */
+                              bufsize = sizeof(buf);
+                              if (ERROR_SUCCESS == RegQueryValueExW(regKey,
+                                L"TZI", 0, &type, (BYTE *)buf, &bufsize))
+                                {
+                                  TZI *tzi = (void*)buf;
+                                  Bias = tzi->Bias;
+                                  StandardBias = tzi->StandardBias;
+                                  DaylightBias = tzi->DaylightBias;
+                                  StandardDate = tzi->StandardDate;
+                                  DaylightDate = tzi->DaylightDate;
+                                }
+                          
+                              /* Set the standard name for the time zone. */
+                              if (wcslen(standardName))
+                                {
+                                  int a, b;
+
+                                  ASSIGN(timeZoneName,
+                                    [NSString stringWithCharacters: standardName
+                                    length: wcslen(standardName)]);
+
+                                  /* Abbr generated here is IMHO
+                                   * a bit suspicous but I kept it */
+                                  for (a = 0, b = 0; standardName[a]; a++)
+                                    {
+                                      if (iswupper(standardName[a]))
+                                        standardName[b++] = standardName[a];
+                                    }
+                                  standardName[b] = L'\0';
+                                  ASSIGN(timeZoneNameAbbr,
+                                    [NSString stringWithCharacters: standardName
+                                    length: wcslen(standardName)]);
+                                }
+
+                              /* Set the daylight savings name
+                               * for the time zone. */
+                              if (wcslen(daylightName))
+                                {
+                                  int a, b;
+
+                                  ASSIGN(daylightZoneName,
+                                    [NSString stringWithCharacters: daylightName
+                                    length: wcslen(daylightName)]);
+
+                                  /* Abbr generated here is IMHO
+                                   * a bit suspicous but I kept it */
+                                  for (a = 0, b = 0; daylightName[a]; a++)
+                                    {
+                                      if (iswupper(daylightName[a]))
+                                        daylightName[b++] = daylightName[a];
+                                    }
+                                  daylightName[b] = L'\0';
+                                  ASSIGN(daylightZoneNameAbbr,
+                                    [NSString stringWithCharacters: daylightName
+                                    length: wcslen(daylightName)]);
+                                }
+                            }
+                          RegCloseKey(regKey);
+                        }			       
+                    }
+                }
+              free(wName);
+            }
+        }
       RegCloseKey(regDirKey);
-  }
+    }
     
   return self;
 }
@@ -2326,7 +2399,7 @@ GSBreakTime(NSTimeInterval when, int *year, int *month, int *day,
         {
 	  return NO;
 	}
-  }
+    }
 
   dow = ((int)((when / 86400.0) + GREGORIAN_REFERENCE)) % 7;
   if (dow < 0)
@@ -2337,13 +2410,13 @@ GSBreakTime(NSTimeInterval when, int *year, int *month, int *day,
       daylightdate = day - dow + DaylightDate.wDayOfWeek;
       maxdate = lastDayOfGregorianMonth(DaylightDate.wMonth, year)-7;
       while (daylightdate > 7)
-        daylightdate-=7;
+        daylightdate -= 7;
       if (daylightdate < 1)
         daylightdate += 7;
-      count=DaylightDate.wDay;
-      while (count>1 && daylightdate < maxdate)
+      count = DaylightDate.wDay;
+      while (count > 1 && daylightdate < maxdate)
         {
-          daylightdate+=7;
+          daylightdate += 7;
           count--;
         }
       if (day > daylightdate)
@@ -2371,13 +2444,13 @@ GSBreakTime(NSTimeInterval when, int *year, int *month, int *day,
       daylightdate = day - dow + StandardDate.wDayOfWeek;
       maxdate = lastDayOfGregorianMonth(StandardDate.wMonth, year)-7;
       while (daylightdate > 7)
-        daylightdate-=7;
+        daylightdate -= 7;
       if (daylightdate < 1)
         daylightdate += 7;
-      count=StandardDate.wDay;
-      while (count>1 && daylightdate < maxdate)
+      count = StandardDate.wDay;
+      while (count > 1 && daylightdate < maxdate)
         {
-          daylightdate+=7;
+          daylightdate += 7;
           count--;
         }
       if (day > daylightdate)
