@@ -11,7 +11,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,7 +24,7 @@
    Boston, MA 02111 USA.
 
    <title>NSTimer class reference</title>
-   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
+   $Date: 2008-06-12 11:44:00 +0100 (Thu, 12 Jun 2008) $ $Revision: 26630 $
    */
 
 #include "config.h"
@@ -99,6 +99,10 @@ static Class	NSDate_class;
   _selector = selector;
   _info = RETAIN(info);
   _repeats = f;
+  if (_repeats == NO)
+    {
+      _interval = 0.0;
+    }
   return self;
 }
 
@@ -192,6 +196,14 @@ static Class	NSDate_class;
     {
       [self invalidate];
     }
+  if (_target != nil)
+    {
+      DESTROY(_target);
+    }
+  if (_info != nil)
+    {
+      DESTROY(_info);
+    }
   RELEASE(_date);
   [super dealloc];
 }
@@ -239,28 +251,6 @@ static Class	NSDate_class;
     {
       [self invalidate];
     }
-  else if (_invalidated == NO)
-    {
-      extern NSTimeInterval GSTimeNow();
-      NSTimeInterval	now = GSTimeNow();
-      NSTimeInterval	nxt = [_date timeIntervalSinceReferenceDate];
-      int		inc = -1;
-
-      while (nxt <= now)		// xxx remove this
-	{
-	  inc++;
-	  nxt += _interval;
-	}
-#ifdef	LOG_MISSED
-      if (inc > 0)
-	{
-	  NSLog(@"Missed %d timeouts at %f second intervals", inc, _interval);
-	}
-#endif
-      RELEASE(_date);
-      _date = [[NSDate_class allocWithZone: NSDefaultMallocZone()]
-	initWithTimeIntervalSinceReferenceDate: nxt];
-    }
 }
 
 /**
@@ -271,14 +261,6 @@ static Class	NSDate_class;
  */
 - (void) invalidate
 {
-  if (_target != nil)
-    {
-      DESTROY(_target);
-    }
-  if (_info != nil)
-    {
-      DESTROY(_info);
-    }
   /* OPENSTEP allows this method to be called multiple times. */
   //NSAssert(_invalidated == NO, NSInternalInconsistencyException);
   _invalidated = YES;
@@ -320,7 +302,8 @@ static Class	NSDate_class;
 }
 
 /**
- * Returns the interval between firings.
+ * Returns the interval between firings, or zero if the timer
+ * does not repeat.
  */
 - (NSTimeInterval) timeInterval
 {
