@@ -9,7 +9,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
    
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,7 +23,7 @@
    */ 
 
 
-#include "GSURLPrivate.h"
+#import "GSURLPrivate.h"
 
 @interface	GSURLDownload : NSObject <NSURLProtocolClient>
 {
