@@ -9,7 +9,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
    
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,24 +22,25 @@
    Boston, MA 02111 USA.
    */ 
 
-#include "GSURLPrivate.h"
+#import "GSURLPrivate.h"
+#import "GSPrivate.h"
 
-#include "Foundation/NSCoder.h"
-#include "Foundation/NSMapTable.h"
-#include "Foundation/NSScanner.h"
-#include "NSCallBacks.h"
-#include "GNUstepBase/GSMime.h"
+#import "Foundation/NSCoder.h"
+#import "Foundation/NSDictionary.h"
+#import "Foundation/NSScanner.h"
+#import "NSCallBacks.h"
+#import "GNUstepBase/GSMime.h"
 
 
 // Internal data storage
 typedef struct {
-  long long			expectedContentLength;
-  NSURL				*URL;
-  NSString			*MIMEType;
-  NSString			*textEncodingName;
-  NSString			*statusText;
-  NSMapTable			*headers;
-  int				statusCode;
+  long long		expectedContentLength;
+  NSURL			*URL;
+  NSString		*MIMEType;
+  NSString		*textEncodingName;
+  NSString		*statusText;
+  NSMutableDictionary	*headers; /* _GSMutableInsensitiveDictionary */
+  int			statusCode;
 } Internal;
  
 typedef struct {
@@ -49,71 +50,92 @@ typedef struct {
 #define	inst	((Internal*)(((priv*)o)->_NSURLResponseInternal))
 
 
-/*
- * Implement map keys for strings with case insensitive comparisons,
- * so we can have case insensitive matching of http headers (correct
- * behavior), but actually preserve case of headers stored and written
- * in case the remote server is buggy and requires particular
- * captialisation of headers (some http software is faulty like that).
- */
-static unsigned int
-_non_retained_id_hash(void *table, NSString* o)
+@implementation	NSURLResponse (Private)
+- (void) _setHeaders: (id)headers
 {
-  return [[o uppercaseString] hash];
+  NSEnumerator	*e;
+  NSString	*v;
+  NSString	*contentLength = nil;
+  GSMimeHeader	*contentType = nil;
+
+  if ([headers isKindOfClass: [NSDictionary class]] == YES)
+    {
+      NSString		*k;
+
+      e = [headers keyEnumerator];
+      while ((k = [e nextObject]) != nil)
+	{
+	  v = [headers objectForKey: k];
+	  [self _setValue: v forHTTPHeaderField: k];
+	}
+    }
+  else if ([headers isKindOfClass: [NSArray class]] == YES)
+    {
+      GSMimeHeader	*h;
+
+      e = [headers objectEnumerator];
+      while ((h = [e nextObject]) != nil)
+        {
+	  NSString	*n = [h namePreservingCase: YES];
+	  NSString	*v = [h fullValue];
+
+	  if ([n caseInsensitiveCompare: @"content-type"] == NSOrderedSame)
+	    {
+	      contentType = h;
+	    }
+	  [self _setValue: v forHTTPHeaderField: n];
+	}
+    }
+
+  if (contentLength == nil)
+    {
+      contentLength = [self _valueForHTTPHeaderField: @"content-length"];
+    }
+  if ([contentLength length] == 0)
+    {
+      this->expectedContentLength = -1;
+    }
+  else
+    {
+      this->expectedContentLength = [contentLength intValue];
+    }
+
+  if (contentType == nil)
+    {
+      GSMimeParser	*p;
+      NSScanner		*s;
+
+      v = [self _valueForHTTPHeaderField: @"content-type"];
+      if (v == nil)
+        {
+	  v = @"text/plain";	// No content type given.
+	}
+      s = [NSScanner scannerWithString: v];
+      p = [GSMimeParser new];
+      contentType = AUTORELEASE([GSMimeHeader new]);
+      [p scanHeaderBody: s into: contentType];
+      RELEASE(p);
+    }
+  ASSIGNCOPY(this->MIMEType, [contentType value]);
+  v = [contentType parameterForKey: @"charset"];
+  ASSIGNCOPY(this->textEncodingName, v);
 }
-
-static BOOL
-_non_retained_id_is_equal(void *table, NSString *o, NSString *p)
-{
-  return ([o caseInsensitiveCompare: p] == NSOrderedSame) ? YES : NO;
-}
-
-typedef unsigned int (*NSMT_hash_func_t)(NSMapTable *, const void *);
-typedef BOOL (*NSMT_is_equal_func_t)(NSMapTable *, const void *, const void *);
-typedef void (*NSMT_retain_func_t)(NSMapTable *, const void *);
-typedef void (*NSMT_release_func_t)(NSMapTable *, void *);
-typedef NSString *(*NSMT_describe_func_t)(NSMapTable *, const void *);
-
-static const NSMapTableKeyCallBacks headerKeyCallBacks =
-{
-  (NSMT_hash_func_t) _non_retained_id_hash,
-  (NSMT_is_equal_func_t) _non_retained_id_is_equal,
-  (NSMT_retain_func_t) _NS_non_retained_id_retain,
-  (NSMT_release_func_t) _NS_non_retained_id_release,
-  (NSMT_describe_func_t) _NS_non_retained_id_describe,
-  NSNotAPointerMapKey
-};
-
-@interface	NSURLResponse (Internal)
-- (void) setStatusCode: (int)code text: (NSString*)text;
-- (void) setValue: (NSString *)value forHTTPHeaderField: (NSString *)field;
-- (NSString *) valueForHTTPHeaderField: (NSString *)field;
-@end
-
-@implementation	NSURLResponse (Internal)
-- (void) setStatusCode: (int)code text: (NSString*)text
+- (void) _setStatusCode: (int)code text: (NSString*)text
 {
   this->statusCode = code;
   ASSIGNCOPY(this->statusText, text);
 }
-- (void) setValue: (NSString *)value forHTTPHeaderField: (NSString *)field
+- (void) _setValue: (NSString *)value forHTTPHeaderField: (NSString *)field
 {
   if (this->headers == 0)
     {
-      this->headers = NSCreateMapTable(headerKeyCallBacks,
-	NSObjectMapValueCallBacks, 8);
+      this->headers = [_GSMutableInsensitiveDictionary new];
     }
-  NSMapInsert(this->headers, (void*)field, (void*)value);
+  [this->headers setObject: value forKey: field];
 }
-- (NSString *) valueForHTTPHeaderField: (NSString *)field
+- (NSString *) _valueForHTTPHeaderField: (NSString *)field
 {
-  NSString	*value = nil;
-
-  if (this->headers != 0)
-    {
-      value = (NSString*)NSMapGet(this->headers, (void*)field);
-    }
-  return value;
+  return [this->headers objectForKey: field];
 }
 @end
 
@@ -156,7 +178,7 @@ static const NSMapTableKeyCallBacks headerKeyCallBacks =
 	    }
 	  else
 	    {
-	      inst->headers = NSCopyMapTableWithZone(this->headers, z);
+	      inst->headers = [this->headers mutableCopy];
 	    }
 	}
     }
@@ -171,10 +193,7 @@ static const NSMapTableKeyCallBacks headerKeyCallBacks =
       RELEASE(this->MIMEType);
       RELEASE(this->textEncodingName);
       RELEASE(this->statusText);
-      if (this->headers != 0)
-        {
-	  NSFreeMapTable(this->headers);
-	}
+      RELEASE(this->headers);
       NSZoneFree([self zone], this);
     }
   [super dealloc];
@@ -247,7 +266,7 @@ static const NSMapTableKeyCallBacks headerKeyCallBacks =
  */
 - (NSString *) suggestedFilename
 {
-  NSString	*disp = [self valueForHTTPHeaderField: @"content-disposition"];
+  NSString	*disp = [self _valueForHTTPHeaderField: @"content-disposition"];
   NSString	*name = nil;
 
   if (disp != nil)
@@ -309,29 +328,12 @@ static const NSMapTableKeyCallBacks headerKeyCallBacks =
 
 - (NSDictionary *) allHeaderFields
 {
-  NSMutableDictionary	*fields;
-
-  fields = [NSMutableDictionary dictionaryWithCapacity: 8];
-  if (this->headers != 0)
-    {
-      NSMapEnumerator	enumerator;
-      NSString		*k;
-      NSString		*v;
-
-      enumerator = NSEnumerateMapTable(this->headers);
-      while (NSNextMapEnumeratorPair(&enumerator, (void **)(&k), (void**)&v))
-	{
-	  [fields setObject: v forKey: k];
-	}
-      NSEndMapTableEnumeration(&enumerator);
-    }
-  return fields;
+  return AUTORELEASE([this->headers copy]);
 }
 
 - (int) statusCode
 {
   return this->statusCode;
 }
-
 @end
 
