@@ -11,7 +11,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,7 +24,7 @@
    Boston, MA 02111 USA.
 
    <title>NSUserDefaults class reference</title>
-   $Date: 2007-11-06 03:27:56 -0700 (Tue, 06 Nov 2007) $ $Revision: 25562 $
+   $Date: 2008-06-12 11:44:00 +0100 (Thu, 12 Jun 2008) $ $Revision: 26630 $
 */
 
 #include "config.h"
@@ -50,7 +50,6 @@
 #include "Foundation/NSSet.h"
 #include "Foundation/NSThread.h"
 #include "Foundation/NSTimer.h"
-#include "Foundation/NSUtilities.h"
 #include "Foundation/NSValue.h"
 #include "Foundation/NSDebug.h"
 #include "GNUstepBase/GSLocale.h"
@@ -198,7 +197,13 @@ static void updateCache(NSUserDefaults *self)
  *     Information retrieved from the GNUstep configuration system.
  *     Usually the system wide and user specific GNUstep.conf files,
  *     or from information compiled in when the base library was
- *     built.
+ *     built.<br />
+ *     In addition to this standard configuration information, this
+ *     domain contains all values from the GlobalDefaults.plist file
+ *     stored in the same directory as the system widw GNUstep.conf
+ *     file.  The GlobalDefaults.plist allows packagers and system
+ *     administrators to provide global defaults settings for all
+ *     users of a particular GNUstep installation.
  *   </desc>
  *   <term><code>NSRegistrationDomain</code> ... volatile</term>
  *   <desc>
@@ -267,14 +272,20 @@ static BOOL setSharedDefaults = NO;     /* Flag to prevent infinite recursion */
     {
       NSDictionary	*regDefs;
 
-      [sharedDefaults synchronize];	// Ensure changes are written.
-      regDefs = RETAIN([sharedDefaults->_tempDomains
-	objectForKey: NSRegistrationDomain]);
       /* To ensure that we don't try to synchronise the old defaults to disk
        * after creating the new ones, remove as housekeeping notification
        * observer.
        */
       [[NSNotificationCenter defaultCenter] removeObserver: sharedDefaults];
+
+      /* Ensure changes are written, and no changes left so we can't end up
+       * writing old changes to the new defaults.
+       */
+      [sharedDefaults synchronize];
+      DESTROY(sharedDefaults->_changedDomains);
+
+      regDefs = RETAIN([sharedDefaults->_tempDomains
+	objectForKey: NSRegistrationDomain]);
       setSharedDefaults = NO;
       DESTROY(sharedDefaults);
       if (regDefs != nil)
@@ -1457,7 +1468,17 @@ static BOOL isLocked = NO;
 
 - (void) unlockDefaultsFile
 {
-  [_fileLock unlock];
+  NS_DURING
+    {
+      [_fileLock unlock];
+    }
+  NS_HANDLER
+    {
+      NSLog(@"Warning ... someone broke our lock (%@) ... and may have"
+        @" interfered with updating defaults data in file.", 
+        [_defaultsDatabase stringByAppendingPathExtension: @"lck"]);
+    }
+  NS_ENDHANDLER
   isLocked = NO;
 }
 
@@ -1684,10 +1705,11 @@ static BOOL isLocked = NO;
 	{
 	  if ((dict = (*pImp)(_persDomains, objectForKeySel, obj)) != nil
 	    || (dict = (*tImp)(_tempDomains, objectForKeySel, obj)) != nil)
-	    (*addImp)(dictRep, addSel, dict);
+            {
+              (*addImp)(dictRep, addSel, dict);
+            }
 	}
-      _dictionaryRep = [dictRep copy];
-      RELEASE(dictRep);
+      _dictionaryRep = [dictRep makeImmutableCopyOnFail: NO];
     }
   rep = RETAIN(_dictionaryRep);
   [_lock unlock];
@@ -1851,13 +1873,16 @@ GSPrivateDefaultsFlag(GSUserDefaultFlagType type)
   return flags[type];
 }
 
-/* FIXME ... Slightly faster than
+/* Slightly faster than
  * [[NSUserDefaults standardUserDefaults] dictionaryRepresentation]
- * but is it really worthwile?
+ * Avoiding the autorelease of the standard defaults turns out to be
+ * a modest but significant gain when making heavy use of methods which
+ * need localisation.
  */
 NSDictionary *GSPrivateDefaultLocale()
 {
-  NSDictionary	*locale;
+  NSDictionary	        *locale;
+  NSUserDefaults        *defs;
 
   if (classLock == nil)
     {
@@ -1868,8 +1893,10 @@ NSDictionary *GSPrivateDefaultLocale()
     {
       [NSUserDefaults standardUserDefaults];
     }
-  locale = [sharedDefaults dictionaryRepresentation];
+  defs = [sharedDefaults retain];
   [classLock unlock];
+  locale = [defs dictionaryRepresentation];
+  [defs release];
   return locale;
 }
 
