@@ -10,7 +10,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
    
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -41,15 +41,6 @@
 
 @implementation NSValueTransformer
 
-NSString * const NSNegateBooleanTransformerName
-  = @"NSNegateBooleanTransformerName";
-NSString * const NSIsNilTransformerName
-  = @"NSIsNilTransformerName";
-NSString * const NSIsNotNilTransformerName
-  = @"NSIsNotNilTransformerName"; 
-NSString * const NSUnarchiveFromDataTransformerName
-  = @"NSUnarchiveFromDataTransformerName";
-
 // non-abstract methods
 
 static NSMutableDictionary *registry = nil;
