@@ -10,7 +10,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,6 +25,10 @@
 
 #include "config.h"
 #include <stdlib.h>
+#ifdef HAVE_ALLOCA_H
+#include <alloca.h>
+#endif
+
 #include "cifframe.h"
 #include "Foundation/NSException.h"
 #include "Foundation/NSData.h"
@@ -465,6 +469,7 @@ cifframe_type(const char *typePtr, const char **advance)
     case _C_VOID: ftype = &ffi_type_void;
       break;
     default:
+      ftype = &ffi_type_void;
       NSCAssert(0, @"Unknown type in sig");
     }
 
@@ -700,7 +705,7 @@ cifframe_do_call (DOContext *ctxt,
 		 @"Will search for arbitrary signature.",
 		 object,
 		 GSNameFromClass(GSObjCIsClass(object) 
-				 ? object : GSObjCClass(object)),
+				 ? object : (id)GSObjCClass(object)),
 		 GSNameFromSelector(selector));
       type = GSTypesFromSelector(selector);
     }
