@@ -9,7 +9,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -34,14 +34,13 @@
 
 #include "config.h"
 #include "GNUstepBase/preface.h"
-#include <mframe.h>
-#include "Foundation/NSObjCRuntime.h"
-#include "Foundation/NSData.h"
-#include "Foundation/NSException.h"
-#include "Foundation/NSDebug.h"
+#ifdef HAVE_ALLOCA_H
+#include <alloca.h>
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
+#include <mframe.h>
 
 /* Deal with strrchr: */
 #if STDC_HEADERS || defined(HAVE_STRING_H)
@@ -60,6 +59,11 @@
 /* memory.h and strings.h conflict on some systems.  */
 #endif /* not STDC_HEADERS and not HAVE_STRING_H */
 
+#include "Foundation/NSObjCRuntime.h"
+#include "Foundation/NSData.h"
+#include "Foundation/NSException.h"
+#include "Foundation/NSDebug.h"
+
 
 
 /* For encoding and decoding the method arguments, we have to know where
@@ -198,11 +202,14 @@ mframe_build_signature(const char *typePtr, int *size, int *narg, char *buf)
 }
 
 
-/*
- *      Step through method encoding information extracting details.
+/* Step through method encoding information extracting details.
+ * If outTypes is non-nul then we copy the argument type into
+ * the buffer as a nul terminated string and use the values in
+ * this buffer as the types in info, rather than pointers to
+ * positions in typePtr
  */
 const char *
-mframe_next_arg(const char *typePtr, NSArgumentInfo *info)
+mframe_next_arg(const char *typePtr, NSArgumentInfo *info, char *outTypes)
 {
   NSArgumentInfo	local;
   BOOL			flag;
@@ -350,7 +357,7 @@ mframe_next_arg(const char *typePtr, NSArgumentInfo *info)
 	    {
 	      typePtr++;
 	    }
-	  typePtr = mframe_next_arg(typePtr, &local);
+	  typePtr = mframe_next_arg(typePtr, &local, 0);
 	  info->size = length * ROUND(local.size, local.align);
 	  info->align = local.align;
 	  typePtr++;	/* Skip end-of-array	*/
@@ -376,7 +383,7 @@ mframe_next_arg(const char *typePtr, NSArgumentInfo *info)
 	   */
 	  if (*typePtr != _C_STRUCT_E)
 	    {
-	      typePtr = mframe_next_arg(typePtr, &local);
+	      typePtr = mframe_next_arg(typePtr, &local, 0);
 	      if (typePtr == 0)
 		{
 		  return 0;		/* error	*/
@@ -391,7 +398,7 @@ mframe_next_arg(const char *typePtr, NSArgumentInfo *info)
 	   */
 	  while (*typePtr != _C_STRUCT_E)
 	    {
-	      typePtr = mframe_next_arg(typePtr, &local);
+	      typePtr = mframe_next_arg(typePtr, &local, 0);
 	      if (typePtr == 0)
 		{
 		  return 0;		/* error	*/
@@ -430,7 +437,7 @@ mframe_next_arg(const char *typePtr, NSArgumentInfo *info)
 	    }
 	  while (*typePtr != _C_UNION_E)
 	    {
-	      typePtr = mframe_next_arg(typePtr, &local);
+	      typePtr = mframe_next_arg(typePtr, &local, 0);
 	      if (typePtr == 0)
 		{
 		  return 0;		/* error	*/
@@ -458,6 +465,17 @@ mframe_next_arg(const char *typePtr, NSArgumentInfo *info)
       return 0;
     }
 
+  /* Copy tye type information into the buffer if provided.
+   */
+  if (outTypes != 0)
+    {
+      unsigned	len = typePtr - info->type;
+
+      strncpy(outTypes, info->type, len);
+      outTypes[len] = '\0';
+      info->type = outTypes;
+    }
+
   /*
    *	May tell the caller if the item is stored in a register.
    */
@@ -495,7 +513,6 @@ mframe_next_arg(const char *typePtr, NSArgumentInfo *info)
   return typePtr;
 }
 
-
 
 /* Return the number of arguments that the method MTH expects.  Note
    that all methods need two implicit arguments `self' and `_cmd'.  */
