@@ -9,7 +9,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -260,15 +260,16 @@ NSString *
 GSPrivateSymbolPath(Class theClass, Category *theCategory)
 {
   unichar buf[MAX_PATH];
+  NSString *s = nil;
   MEMORY_BASIC_INFORMATION memInfo;
   NSCAssert(!theCategory, @"GSPrivateSymbolPath doesn't support categories");
 
   VirtualQueryEx(GetCurrentProcess(), theClass, &memInfo, sizeof(memInfo));
   if (GetModuleFileNameW(memInfo.AllocationBase, buf, sizeof(buf)))
     {
-      return [NSString stringWithCharacters: buf length: wcslen(buf)];
+      s = [NSString stringWithCharacters: buf length: wcslen(buf)];
     }
-  return 0;
+  return s;
 }
 #else
 NSString *
