@@ -1,7 +1,19 @@
+/* Test/example program for the base library
+
+   Copyright (C) 2005 Free Software Foundation, Inc.
+   
+  Copying and distribution of this file, with or without modification,
+  are permitted in any medium without royalty provided the copyright
+  notice and this notice are preserved.
+
+   This file is part of the GNUstep Base Library.
+*/
 #include <Foundation/NSAutoreleasePool.h>
 #include <Foundation/NSFileHandle.h>
+#include <Foundation/NSFileManager.h>
 #include <Foundation/NSData.h>
 #include <Foundation/NSString.h>
+#include <Foundation/NSURL.h>
 #include <assert.h>
 
 int
@@ -26,7 +38,7 @@ main ()
   assert(dst != nil);
 
   d0 = [[src readDataToEndOfFile] retain];
-  [dst writeData:d0];
+  [(NSFileHandle*)dst writeData: d0];
   [src release];
   [dst release];
   [pool release];
@@ -44,5 +56,11 @@ main ()
   else
     printf("Test failed\n");
 
+  pool = [[NSAutoreleasePool alloc] init];
+  src = [NSURL URLWithString: @"http://www.w3.org/index.html"];
+  d0 = [src resourceDataUsingCache: NO];
+  NSLog(@"Data is %@", d0);
+  [pool release];
+  
   exit (0);
 }
