@@ -10,11 +10,11 @@
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
-   as published by the Free Software Foundation; either version 2
-   of the License, or (at your option) any later version.
+   as published by the Free Software Foundation; either
+   version 3 of the License, or (at your option) any later version.
 
    You should have received a copy of the GNU General Public
-   License along with this program; see the file COPYING.LIB.
+   License along with this program; see the file COPYINGv3.
    If not, write to the Free Software Foundation,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
@@ -2505,7 +2505,7 @@ static NSString		*mainFont = nil;
 	    }
 	  if ([gvrem length] > 0)
 	    {
-	      [buf appendString: @" deprecated for removal at "];
+	      [buf appendString: @" Likely to be changed/moved/removed at "];
 	      [buf appendString: gvrem];
 	    }
 	}
@@ -2531,7 +2531,7 @@ static NSString		*mainFont = nil;
       [buf appendString: @"<br />\n"];
       if ([gvrem length] > 0)
 	{
-	  [buf appendString: @" deprecated for removal at "];
+          [buf appendString: @" Likely to be changed/moved/removed at "];
 	  [buf appendString: gvrem];
 	}
       [buf appendString:@"</div>\n"];
