@@ -1,5 +1,7 @@
-/* The GNUstep HTML Linker
-   Copyright (C) 2002 Free Software Foundation, Inc.
+/** The GNUstep HTML Linker
+
+   <title>HTMLLinker. A tool to fix up href references in html files</title>
+   Copyright (C) 2002,2007 Free Software Foundation, Inc.
 
    Written by: Nicola Pero <nicola@brainstorm.co.uk>
    Date: January 2002
@@ -8,13 +10,14 @@
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
-   as published by the Free Software Foundation; either version 2
-   of the License, or (at your option) any later version.
+   as published by the Free Software Foundation; either
+   version 3 of the License, or (at your option) any later version.
 
    You should have received a copy of the GNU General Public
-   License along with this program; see the file COPYING.LIB.
+   License along with this program; see the file COPYINGv3.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+
    */
 
 /*
@@ -31,6 +34,8 @@ static NSFileManager *fileManager = nil;
 /* [[NSFileManager defaulManager] currentDirectoryPath]  */
 static NSString *currentPath = nil;
 
+static int verbose = 0;
+
 /* Enumerate all .html (or .htmlink) files in a directory and
    subdirectories.  */
 @interface HTMLDirectoryEnumerator : NSEnumerator
@@ -99,6 +104,12 @@ static NSString *currentPath = nil;
 	  found = YES;
 	}
       
+      if ([[[e fileAttributes] fileType] isEqual: NSFileTypeDirectory]
+      	  && verbose)
+        {
+	  GSPrintf(stdout, @"    traversing %@\n", s);
+	}
+
       if (found)
 	{
 	  if (returnsAbsolutePaths)
@@ -807,13 +818,12 @@ static NSString *currentPath = nil;
 	       mapping if we can match.  */
 	    NSEnumerator *en = [pathMappings keyEnumerator];
 	    NSString *key;
-	    
 	    while ((key = [en nextObject]))
 	      {
 		if ([filePath hasPrefix: key])
 		  {
 		    NSString *value = [pathMappings objectForKey: key];
-		    
+
 		    filePath = [filePath substringFromIndex: [key length]];
 		    filePath = [value stringByAppendingPathComponent: 
 					filePath];
@@ -981,14 +991,10 @@ static NSString *currentPath = nil;
       /* Not found - leave it unfixed.  */
       if (file == nil)
 	{
-	  if (warn)
+	  if (warn && [fileLink length] > 0)
 	    {
-	      NSString *m;
-	  
-	      m = [NSString stringWithFormat: 
-			      @"%@: Unresolved reference to '%@'\n", 
-			    logFile, nameLink];
-	      fprintf (stderr, [m lossyCString]);
+	      GSPrintf(stderr, @"%@: Unresolved reference to '%@'\n", 
+			    logFile, nameLink);
 	    }
 	  
 	  relocatedFileLink = fileLink;
@@ -1013,10 +1019,14 @@ static NSString *currentPath = nil;
 
 @end
 
-static NSString *build_relocation_table_for_directory (NSString *dir)
+static NSDictionary *
+build_relocation_table_for_directory (NSString *dir)
 {
   BOOL isDir;
 
+  if (verbose)
+    GSPrintf(stdout, @"  Building relcation table for %@\n", dir);
+
   /* Check if it's a directory; if it is, enumerate all HTML files
      inside it, and add all of them.  */
   if (![fileManager fileExistsAtPath: dir  isDirectory: &isDir])
@@ -1068,22 +1078,20 @@ static NSString *build_relocation_table_for_directory (NSString *dir)
 	      [relocationTable setObject: filename  forKey: name];
 	    }
 	}
-      return [relocationTable description];
+      return relocationTable;
     }
 }
 
 
 static void print_help_and_exit ()
 {
-  printf ("GNUstep HTMLLinker (gnustep-base version %d.%d.%d)\n", 
-	  GNUSTEP_BASE_MAJOR_VERSION,
-	  GNUSTEP_BASE_MINOR_VERSION,
-	  GNUSTEP_BASE_SUBMINOR_VERSION);
+  printf ("GNUstep HTMLLinker\n");
   printf ("Usage: HTMLLinker [options] input_files [-l relocation_file] [-d destination_file]\n");
   printf ("Multiple input files, and multiple -l and -d options are allowed.\n");
   printf (" `options' include:\n");
   printf ("  --help: print this message;\n");
   printf ("  --version: print version information;\n");
+  printf ("  --verbose: print information while processing;\n");
   printf ("  -Warn NO: do not print warnings about unresolved links;\n");
   printf ("  -LinksMarker xxx: only fixup links with attribute rel=xxx;\n");
   printf ("  -FixupAllLinks YES: attempt to fixup all links (not only ones with the marker);\n");
@@ -1106,7 +1114,6 @@ static void print_version_and_exit ()
 
 int main (int argc, char** argv, char** env)
 {
-  CREATE_AUTORELEASE_POOL(pool);
   NSUserDefaults *userDefs;
   NSArray *args;
   NSMutableArray *inputFiles;
@@ -1114,6 +1121,7 @@ int main (int argc, char** argv, char** env)
   BOOL warn, fixupAllLinks;
   NSString *linksMarker;
   HTMLLinker *linker;
+  CREATE_AUTORELEASE_POOL(pool);
 
 #ifdef GS_PASS_ARGUMENTS
   [NSProcessInfo initializeWithArguments:argv count:argc environment:env];
@@ -1150,7 +1158,7 @@ int main (int argc, char** argv, char** env)
 
     if (relFile != nil)
       {
-	NSString *table;
+	NSDictionary *table;
 	NSString *outputFile;
 
 	outputFile = [relFile stringByAppendingPathComponent: 
@@ -1221,25 +1229,27 @@ int main (int argc, char** argv, char** env)
   for (i = 1; i < count; i++)
     {
       NSString *arg = [args objectAtIndex: i];
-      
-      if ([arg hasPrefix: @"--"])
+      if ([arg characterAtIndex: 0] == '-')
 	{
-	  if ([arg isEqualToString: @"--help"])
+	  NSString *opt;
+	  opt = ([arg characterAtIndex: 1] == '-') ?
+	      [arg substringFromIndex: 2] : [arg substringFromIndex: 1];
+	  if ([opt isEqualToString: @"help"]
+	      || [opt isEqualToString: @"h"])
 	    {
 	      print_help_and_exit ();
 	    }
-	  else if ([arg isEqualToString: @"--version"])
+	  else if ([opt isEqualToString: @"version"]
+		   || [opt isEqualToString: @"V"])
 	    {
 	      print_version_and_exit ();
 	    }
-	  else
+	  else if ([opt isEqualToString: @"verbose"]
+		   || [opt isEqualToString: @"v"])
 	    {
-	      /* Ignore it for future expansions.  */
+	      verbose++;
 	    }
-	}
-      else if ([arg hasPrefix: @"-"])
-	{
-	  if ([arg isEqualToString: @"-d"])
+	  else if ([opt isEqualToString: @"d"])
 	    {
 	      if ((i + 1) < count)
 		{
@@ -1252,7 +1262,7 @@ int main (int argc, char** argv, char** env)
 		  NSLog (@"Missing argument to -d");
 		}
 	    }
-	  else if ([arg isEqualToString: @"-l"])
+	  else if ([opt isEqualToString: @"l"])
 	    {
 	      if ((i + 1) < count)
 		{
@@ -1322,6 +1332,8 @@ int main (int argc, char** argv, char** env)
       HTMLParser *parser;
 
       inputFile = [inputFiles objectAtIndex: i];
+      if (verbose)
+        GSPrintf(stdout, @"  %@\n", inputFile);
       inputFileContents = [NSString stringWithContentsOfFile: inputFile];
       
       parser = [[HTMLParser alloc] initWithCode: inputFileContents];
