@@ -6,13 +6,13 @@
 
    This file is part of the GNUstep Project
 
-   This library is free software; you can redistribute it and/or
+   This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
-   as published by the Free Software Foundation; either version 2
-   of the License, or (at your option) any later version.
+   as published by the Free Software Foundation; either
+   version 2 of the License, or (at your option) any later version.
 
    You should have received a copy of the GNU General Public
-   License along with this library; see the file COPYING.LIB.
+   License along with this program; see the file COPYING.
    If not, write to the Free Software Foundation,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
@@ -242,7 +242,7 @@ ihandler(int sig)
 @interface	GDNCObserver : NSObject
 {
 @public
-  unsigned		observer;
+  unsigned long		observer;
   NSString		*notificationName;
   NSString		*notificationObject;
   NSString		*selector;
@@ -472,6 +472,7 @@ ihandler(int sig)
 	  name = [service stringByAppendingFormat: @"-%@", name];
 	  if ([ns registerPort: port forName: name] == NO)
 	    {
+              NSLog(@"gdnc - failed to register as %@", name);
 	    }
 	}
       a = [host addresses];
@@ -483,6 +484,7 @@ ihandler(int sig)
 	  name = [service stringByAppendingFormat: @"-%@", name];
 	  if ([ns registerPort: port forName: name] == NO)
 	    {
+              NSLog(@"gdnc - failed to register as %@", name);
 	    }
 	}
     }
